/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include "grafikausg.h"

#ifndef GESCHENKE_H
#define GESCHENKE_H 1

class Punkte
  {
	protected:
		GrafikAusgabe 	*kanz, *ganz, *kfertig, *gfertig;
		int 		kmult, gmult;
	public:
		virtual int Getkanz() const = 0;
		virtual int Getganz() const = 0;
	 	virtual int Getkfertig() const = 0;
		virtual int Getgfertig() const = 0;
		virtual int Getkmult() const = 0;
		virtual int Getgmult() const = 0;
  };

class Feldpunkte : public Punkte
  {
	public:
		Feldpunkte(int klein = 0, int gross = 0, 
			   int kanzfertig = 0, int ganzfertig = 0);
		Feldpunkte(const Punkte& p);
		virtual ~Feldpunkte();
		void SetMult(int kmultvalue, int smultvalue);
		void Clear();
		int  IsClear();

		int Getkanz() const 
			{ return ((kanz) ? (*kanz + 0) : 0); };
		int Getganz() const 
			{ return ((ganz) ? (*ganz + 0) : 0); };
		int Getkfertig() const 
			{ return ((kfertig) ? (*kfertig + 0) : 0); };
		int Getgfertig() const 
			{ return ((gfertig) ? (*gfertig + 0) : 0); };
		int Getkmult() const
			{ return kmult; };
		int Getgmult() const
			{ return gmult; };
		GrafikAusgabe* GetPkanz() const
			{ return kanz; };
		GrafikAusgabe* GetPganz() const
			{ return ganz; };
		GrafikAusgabe* GetPkfertig() const
			{ return kfertig; };
		GrafikAusgabe* GetPgfertig() const
			{ return gfertig; };

		const Feldpunkte& operator = (const Feldpunkte& newfp); 
  };

class Geschenke : public Punkte
  {
	private:
		int punktanz;
	public: 
		Geschenke(int kanzfertig, int ganzfertig);
		virtual ~Geschenke();
		int  Fertig();
		void TriggerPunktAnz() { punktanz = punktanz ? 0 : 1; };
		int  CalcPunkte();
		void DrawGeschenke();
		const Geschenke& operator += (const Punkte& addpunkte);

		int Getkanz() const 
			{ return ((kanz) ? (*kanz + 0) : 0); };
		int Getganz() const 
			{ return ((ganz) ? (*ganz + 0) : 0); };
		int Getkfertig() const 
			{ return ((kfertig) ? (*kfertig + 0) : 0); };
		int Getgfertig() const 
			{ return ((gfertig) ? (*gfertig + 0) : 0); };
		int Getkmult() const
			{ return 10; };
		int Getgmult() const
			{ return 10; };
  };

#endif

