/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#ifndef GRAFIKAUSGABE_H
#define GRAFIKAUSGABE_H 1

#include <iostream>

typedef  unsigned int fontchar[2];

class GrafikAusgabe
  {		
	private:
		char  		alpha;
		const char  	*text;
		int   		wert;
		float 		posx, posy, posz, size;
		float 		red, green, blue;
				//0.0 - 1.0, black-white
		fontchar 	fontdata[128];
		fontchar 	numberfont[10];

		void  		Initfont();
		void  		DrawChar(fontchar letter) const;
		void  		DrawGlutChar(char c) const;
		int		Compare(const GrafikAusgabe& rightga) const;

	public:
		GrafikAusgabe(float neux=0, float neuy=0, float neuz=0, 
				int neuwert=0)
		 : wert(neuwert), posx(neux), posy(neuy), posz(neuz) 
			{ size = 0.5;  alpha = 0; SetColor(); text = 0; 
			  Initfont(); };
		~GrafikAusgabe() {};

		void SetSize(float neusize) 
			{ if (neusize > 0.0f) size = neusize; };
		float GetSize() const
			{ return size; };
		void SetColor(float nred=1.0, float ngreen=1.0, 
				float nblue=1.0) 
			{  red = nred; green = ngreen; blue = nblue; };
		void GetColor(float *thisred, float *thisgreen, 
				float *thisblue) const
			{ thisred   = const_cast<float*> (&red); 
			  thisgreen = const_cast<float*> (&green); 
			  thisblue  = const_cast<float*> (&blue); };
		void SetText(const char *neutext) 
			{ text = neutext; alpha = 1; }; 
		void GetText(char* thistext, char* thisalpha) const
			{ thistext  = const_cast<char*> (text); 
			  thisalpha = const_cast<char*> (&alpha); };
		void SetZahl() { alpha = 0; };

		void SetWert(int neuwert) { wert = neuwert; };
		void AddWert(int addw) { wert = wert + addw; };
		int  GetWert() const
			{ return wert; };
		void AddPos(float addx=0, float addy=0, float addz=0)
			{ posx = posx+addx; 
			  posy = posy+addy; 
			  posz = posz+addz; };
		void NeuPos(float neux=0, float neuy=0, float neuz=0)
			{ posx = neux; 
			  posy = neuy; 
			  posz = neuz; };
		void GetPos(float* thisx, float* thisy, float* thisz) const
			{ thisx = const_cast<float*> (&posx); 
			  thisy = const_cast<float*> (&posy); 
			  thisz = const_cast<float*> (&posz); };
		void DrawAusgabe();

		const GrafikAusgabe& operator = (const GrafikAusgabe& newga);
		const GrafikAusgabe& operator = (const int neuwert)
			{ SetWert(neuwert);  return *this; }; 
		const GrafikAusgabe& operator = (const char *neutext)
			{ SetText(neutext);  return *this; };
		const GrafikAusgabe& operator += (const int addwert)
			{ AddWert(addwert);  return *this; }; 
		const GrafikAusgabe& operator -= (const int addwert)
			{ AddWert(-addwert); return *this; }; 
		const GrafikAusgabe& operator *= (const int multwert)
			{ wert *= multwert;  return *this; }; 

		int operator + (const int rightv) const
			{ return ( wert + rightv ); }; 
		int operator - (const int rightv) const
			{ return ( wert - rightv ); }; 
		int operator * (const int rightv) const
			{ return ( wert * rightv ); }; 

		friend int operator == (const GrafikAusgabe& leftga,
					const GrafikAusgabe& rightga)
			{ return ( leftga.Compare(rightga) == 0); };
		friend int operator != (const GrafikAusgabe& leftga,
					const GrafikAusgabe& rightga)
			{ return (leftga.Compare(rightga) != 0); };
		friend int operator <  (const GrafikAusgabe& leftga,
					const GrafikAusgabe& rightga)
			{ return (leftga.Compare(rightga) <  0); };
		friend int operator >  (const GrafikAusgabe& leftga,
					const GrafikAusgabe& rightga)
			{ return (leftga.Compare(rightga) >  0); };
		friend int operator <= (const GrafikAusgabe& leftga,
					const GrafikAusgabe& rightga)
			{ return (leftga.Compare(rightga) <= 0); };
		friend int operator >= (const GrafikAusgabe& leftga,
					const GrafikAusgabe& rightga)
			{ return (leftga.Compare(rightga) >= 0); };
  };
		   
#endif
