/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#ifndef _LISTEN_H
#define _LISTEN_H 1

#include <stdio.h>				// needed for NULL

using namespace std;

typedef		int	Valuetype;		// should be a number

class Simplelist
  {
	protected:
		Valuetype	value;
		Simplelist	*next;
	public:
		Simplelist(Valuetype newvalue = 0) : 
			value(newvalue), next(NULL) {};
		~Simplelist();
		void InsLast(Simplelist* &last, const Valuetype newvalue);
		void InsFirst(const Valuetype newvalue); 	// Inserthere
		int  DelFirst();
		int  DelNext();
		
		void FindValue(const Valuetype newvalue, 
				Simplelist* &newlist) const;
					// return one before to make DelNext
		void GetNext(Simplelist* &thisnext) const;
		void SetNext(Simplelist* const newnext);
		void GetValue(Valuetype &thisvalue) const;
		void SetValue(const Valuetype newvalue);
  };

template<class Texttype>
class Objpointerlist : public Simplelist
  {
	protected:
		Texttype	*textstring;
		Objpointerlist	*next;
	public:
		Objpointerlist(Texttype* const newtext = NULL) : 
			Simplelist(0), textstring(newtext), next(NULL) {;};
		~Objpointerlist();
		void InsLast(Objpointerlist* &last, Texttype* const newtext);
		void InsFirst(Texttype* const newtext);
		int  DelFirst();
		int  DelNext();
		
		void FindText(const Texttype newtext, 
				Objpointerlist* &newlist) const;
		void FindValue(const Valuetype newvalue, 
				Objpointerlist* &newlist) const;
					// return one before to make DelNext
		void GetText(Texttype* &thistext) const;
		void SetText(Texttype* const newtext);
		friend istream& operator >> <>(istream& instream, 
				     Objpointerlist<Texttype>& thisobjl);
		friend ostream& operator << <>(ostream& outstream, 
				     const Objpointerlist<Texttype>& thisobjl);

		void GetNext(Objpointerlist* &thisnext) const;
		void SetNext(Objpointerlist* const newnext);
		void GetValue(Valuetype &thisvalue) const;
		void SetValue(const Valuetype newvalue);
  };

class Doublelist : public Simplelist
  {
	protected:
		Simplelist	*subbegin, *subend;
		Doublelist	*prev, *next;
	public:
		Doublelist(	Simplelist* const neubegin = NULL, 
				Simplelist* const neuend = NULL) : 
			Simplelist(0), subbegin(neubegin), subend(neuend),
			prev(NULL), next(NULL) {;};
		~Doublelist();	
		void InsLast(	Doublelist* &last, 
				Simplelist* const newbegin, 
				Simplelist* const newend);
		void InsFirst(	Simplelist* const newbegin, 
				Simplelist* const newend);
		int  DelFirst();
		int  DelNext();

		void FindBegin(	Simplelist* const newbegin, 
				Doublelist* &newlist) const;
		void FindEnd(Simplelist* const newend, 
				Doublelist* &newlist) const;
		void FindValue(const Valuetype newvalue, 
				Doublelist* &newlist) const;
					// return one before to make DelNext

		void GetBegin(Simplelist* &thisbegin) const;
		void GetEnd(Simplelist* &thisend) const;
		void SetBegin(Simplelist* const newbegin);
		void SetEnd(Simplelist* const newend);
		void GetPrev(Doublelist* &thisprev) const;
		void SetPrev(Doublelist* const newprev);
		void GetNext(Doublelist* &thisnext) const;
		void SetNext(Doublelist* const newnext);
  };

#include "../src/objpointerlist.cpp"

#endif

