/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <GL/glut.h>
#include "../include/konstanten.h"
#include "../include/geschenke.h"

//---Klasse Feldpunkte----------------------------

Feldpunkte::Feldpunkte(int klein = 0, int gross = 0, 
		       int kanzfertig = 0, int ganzfertig = 0)
{
  kmult = 10; gmult = 10;
  kanz    = new GrafikAusgabe();
  ganz    = new GrafikAusgabe();
  kfertig = new GrafikAusgabe();
  gfertig = new GrafikAusgabe();
  *kanz = klein; *ganz = gross; *kfertig = kanzfertig; *gfertig = ganzfertig;
};

Feldpunkte::Feldpunkte(const Punkte& p)
{
  Feldpunkte(0, 0, 0, 0);
};

Feldpunkte::~Feldpunkte()
{
  delete kanz;
  delete ganz;
  delete kfertig;
  delete gfertig;
};

void Feldpunkte::SetMult(int kmultvalue, int gmultvalue)
{
  kmult = kmultvalue; gmult = gmultvalue;
};

void Feldpunkte::Clear()
{
  *kanz = 0; *ganz = 0; *kfertig = 0; *gfertig = 0;
  *kanz = " "; *ganz = " "; *kfertig = " "; *gfertig = " ";
  kmult = 10; gmult = 10;
};

int Feldpunkte::IsClear()
{
  int boolvar;

  boolvar = (*kanz + 0 == 0) && (*ganz + 0 == 0);
  boolvar = boolvar && (*kfertig + 0 == 0) && (*gfertig + 0 == 0);
  boolvar = boolvar && ( kmult == 10 ) && ( gmult == 10 ); 
  return boolvar;
};

const Feldpunkte& Feldpunkte::operator = (const Feldpunkte& newfp)
{
  *kanz    = *(newfp.GetPkanz());
  *ganz    = *(newfp.GetPganz());
  *kfertig = *(newfp.GetPkfertig());
  *gfertig = *(newfp.GetPgfertig());
  kmult = newfp.Getkmult();
  gmult = newfp.Getgmult();
  return *this;
};

//---Klasse Geschenke-----------------------------

Geschenke::Geschenke(int kanzfertig, int ganzfertig)
{
  kanz = new GrafikAusgabe(-3/1.0f, -2.25, 0);
  kanz->SetSize(0.3f);
  kanz->SetColor(C_SMALL_P[0], C_SMALL_P[1], C_SMALL_P[2]);
  kfertig = new GrafikAusgabe(-3/2.0f, -2.25, 0, kanzfertig);
  kfertig->SetSize(0.3f);
  kfertig->SetColor(C_SMALL_F[0], C_SMALL_F[1], C_SMALL_F[2]);
  ganz = new GrafikAusgabe(-3/1.0f, -2.75, 0);
  ganz->SetSize(0.3f);
  ganz->SetColor(C_BIG_P[0], C_BIG_P[1], C_BIG_P[2]);
  gfertig = new GrafikAusgabe(-3/2.0f, -2.75, 0, ganzfertig);
  gfertig->SetSize(0.3f);
  gfertig->SetColor(C_BIG_F[0], C_BIG_F[1], C_BIG_F[2]);

  this->punktanz = 1;
}

Geschenke::~Geschenke()
{
  delete kanz; 
  delete ganz;
  delete kfertig;
  delete gfertig;
}

int Geschenke::Fertig()
{
  int erg=0;
  
  if ( (*kanz  >= *kfertig ) && (*ganz >= *gfertig) ) erg = 1;
  return erg;
};

int Geschenke::CalcPunkte()
{
  int erg;
  
  erg = (*kanz) + (*ganz) *100 ;
  return erg;
}

void Geschenke::DrawGeschenke()
{
  if (punktanz)
    {
	kanz->DrawAusgabe();
	kfertig->DrawAusgabe();
	ganz->DrawAusgabe();
	gfertig->DrawAusgabe();
    }
}

const Geschenke& Geschenke::operator += (const Punkte& addpunkte)
{
  *kanz += addpunkte.Getkanz() + 0;
  *ganz += addpunkte.Getganz() + 0;
  if (( addpunkte.Getkanz() == 9) && ( addpunkte.Getganz() == 9))
    {
	(*kanz) *= 2; (*ganz) *= 2;
    }
  *kfertig = (*kfertig + (addpunkte.Getkfertig() + 0)) 
		* addpunkte.Getkmult() / 10;
  *gfertig = (*gfertig + (addpunkte.Getgfertig() + 0)) 
		* addpunkte.Getgmult() / 10;

  return *this;
};

