/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <GL/glut.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "../include/grafikausg.h"
#include "../include/konstanten.h"

//---Klasse GrafikAusgabe-------------------------

void GrafikAusgabe::Initfont()
{
	// Punkte  0x123AC980 -> Linie von 1 nach 2,3,10,12,9,8,0  
	//	  .  .  .  1 2 3    0=Unterbrechung
	//	  .  .  .  4 5 6	max. 16 Punkte	
	//	  .  .  .  7 8 9
	//	  .  .  .  A B C
	//	  .  .  .  D E F

  short i;

  for(i=0;  i<10; i++) { numberfont[i][0] = 0; numberfont[i][1] = 0; };
  for(i=0; i<128; i++) {   fontdata[i][0] = 0;   fontdata[i][1] = 0; };

  numberfont[0][0]=0x13FD1000; numberfont[1][0]=0x3F000000; 
  numberfont[2][0]=0x1397DF00; numberfont[3][0]=0x13979FD0; 
  numberfont[4][0]=0x17939F00; numberfont[5][0]=0x3179FD00;			
  numberfont[6][0]=0x3179FD70; numberfont[7][0]=0x13F00000; 
  numberfont[8][0]=0x71397DF9; numberfont[9][0]=0x9713FD00;

  fontdata['\0'][0]=0;			// nichts, wie Leerzeichen
  fontdata['\001'][0]=0x13FD107B;  fontdata['\001'][1]=0x90580000; 
  fontdata['\002'][0]=0x13FD107B;  fontdata['\002'][1]=0x90287964;
  fontdata['\003'][0]=0x847E9680;
  fontdata['\004'][0]=0x27E92E79;
  fontdata['\005'][0]=0xEBADBCFB;  fontdata['\005'][1]=0x759B0000;
  fontdata['\006'][0]=0xEBA75259;  fontdata['\006'][1]=0xCB000000;
  fontdata['\a'][0]=0;			// nichts, wie Leerzeichen (kein beep bei jedem Update)
  fontdata['\b'][0]=0;			// nichts, wie Leerzeichen
  fontdata['\t'][0]=0;			// auf nchste 8er Position
  fontdata['\n'][0]=0;			// auf Anfang der nchsten Zeile, kein Lschen, da zeilenweise Verarbeitung
  fontdata['\v'][0]=0;			// nichts, wie Leerzeichen (vertical tab ?)
  fontdata['\f'][0]=0;			// nichts, wie Leerzeichen (form feed ?)
  fontdata['\r'][0]=0;			// wie \n
  fontdata['\016'][0]=0xADEA13CB;  fontdata['\016'][1]=0x96400000;
  fontdata['\017'][0]=0x257BEB95;  fontdata['\017'][1]=0x0460AC00;
  fontdata['\020'][0]=0xA94A0000;
  fontdata['\021'][0]=0xC67C0000;
  fontdata['\022'][0]=0x246205B0;  fontdata['\022'][1]=0xAECA0000;
  fontdata['\023'][0]=0x170AD028;  fontdata['\023'][1]=0x0BE00000;
  fontdata['\024'][0]=0xF32E8420;
  fontdata['\025'][0]=0x319B057F;  fontdata['\025'][1]=0xD0000000;
  fontdata['\026'][0]=0x79CA7000;
  fontdata['\027'][0]=0x246205B0;  fontdata['\027'][1]=0xAECA0DF0;
  fontdata['\030'][0]=0x62465E00;	// Cursor nicht bewegt
  fontdata['\031'][0]=0x2BACEA00;	// Cursor nicht bewegt
  fontdata['\032'][0]=0x78B59B00;	// Cursor nicht bewegt
  fontdata['\033'][0]=0x98B57B00;	// Cursor nicht bewegt
  fontdata['\034'][0]=0;		// nichts, wie Leerzeichen
  fontdata['\035'][0]=0;		// nichts, wie Leerzeichen
  fontdata['\036'][0]=0;		// nichts, wie Leerzeichen
  fontdata['\037'][0]=0;		// nichts, wie Leerzeichen
  fontdata[' '][0]=0xF0000000;
  fontdata['!'][0]=0x82380BE0;  
  fontdata['\"'][0]=0x14025000; 
  fontdata['#'][0]=0x5D06E079;  fontdata['#'][1]=0x0AC00000;
  fontdata['$'][0]=0x32478CED;  fontdata['$'][1]=0x02E00000;  
  fontdata['%'][0]=0x1254103D;  fontdata['%'][1]=0x0BCFEB00; 
  fontdata['&'][0]=0x9EA324F0;  
  fontdata['\''][0]=0x36800000;  
  fontdata['('][0]=0x24AE0000; 
  fontdata[')'][0]=0x26CE0000;
  fontdata['*'][0]=0xA850798C; 
  fontdata['+'][0]=0x5B079000; 
  fontdata[','][0]=0x9CE00000;
  fontdata['-'][0]=0x79000000; 
  fontdata['.'][0]=0xEF000000;
  fontdata['/'][0]=0xD3000000;
  fontdata['0'][0]=numberfont[0][0];
  fontdata['1'][0]=numberfont[1][0];    
  fontdata['2'][0]=numberfont[2][0];
  fontdata['3'][0]=numberfont[3][0]; 
  fontdata['4'][0]=numberfont[4][0]; 
  fontdata['5'][0]=numberfont[5][0];
  fontdata['6'][0]=numberfont[6][0]; 
  fontdata['7'][0]=numberfont[7][0]; 
  fontdata['8'][0]=numberfont[8][0];
  fontdata['9'][0]=numberfont[9][0]; 
  fontdata[':'][0]=0x580BE000;    	
  fontdata[';'][0]=0x890CE000;
  fontdata['<'][0]=0x67C00000; 
  fontdata['='][0]=0x460AC000; 
  fontdata['>'][0]=0x49A00000;
  fontdata['?'][0]=0x42680BE0; 
  fontdata['@'][0]=0x98BC64DF; 
  fontdata['A'][0]=0xD72979F0;
  fontdata['B'][0]=0x126878CE;  fontdata['B'][1]=0xD1000000; 
  fontdata['C'][0]=0x324AEF00; 
  fontdata['D'][0]=0x126CED10;
  fontdata['E'][0]=0x31787DF0; 
  fontdata['F'][0]=0x31787D00; 
  fontdata['G'][0]=0x324AEF98;
  fontdata['H'][0]=0x1D793F00; 
  fontdata['I'][0]=0x132EDF00; 
  fontdata['J'][0]=0x13CEA000;
  fontdata['K'][0]=0x1D737F00; 
  fontdata['L'][0]=0x1DF00000; 
  fontdata['M'][0]=0xD183F000;
  fontdata['N'][0]=0xD1F30000; 
  fontdata['O'][0]=0x24AEC620; 
  fontdata['P'][0]=0xD1268700;
  fontdata['Q'][0]=0x24AEC620; fontdata['Q'][1]=0x8F000000; 
  fontdata['R'][0]=0xD12687F0; 
  fontdata['S'][0]=0x32478CED;
  fontdata['T'][0]=0x132E0000; 
  fontdata['U'][0]=0x1DF30000; 
  fontdata['V'][0]=0x1E300000;
  fontdata['W'][0]=0x1D8F3000; 
  fontdata['X'][0]=0x1F03D000; 
  fontdata['Y'][0]=0x1838E000; 
  fontdata['Z'][0]=0x13DF0000; 
  fontdata['['][0]=0x21DE0000; 
  fontdata['\\'][0]=0xF1000000;
  fontdata[']'][0]=0x23FE0000; 
  fontdata['^'][0]=0x42600000; 
  fontdata['_'][0]=0xDF000000; 
  fontdata['`'][0]=0x25900000;  
  fontdata['a'][0]=0x98AEC9F0;
  fontdata['b'][0]=0x1DEC8700;
  fontdata['c'][0]=0x98AEF000; 
  fontdata['d'][0]=0x3FEA8900;
  fontdata['e'][0]=0xAC87AEF0; 
  fontdata['f'][0]=0x35E07900; 
  fontdata['g'][0]=0x97BC9FE0;
  fontdata['h'][0]=0x1DA8CF00; 
  fontdata['i'][0]=0x5608EDF0; 
  fontdata['j'][0]=0x5609CEA0;
  fontdata['k'][0]=0x1DA9AF00; 
  fontdata['l'][0]=0x2EC00000; 
  fontdata['m'][0]=0xD7A8EB9F;
  fontdata['n'][0]=0x7DA8CF00; 
  fontdata['o'][0]=0x8AEC8000; 
  fontdata['p'][0]=0xD459B700;
  fontdata['q'][0]=0x657B96F0;
  fontdata['r'][0]=0xD7A89000; 
  fontdata['s'][0]=0x98ACED00;
  fontdata['t'][0]=0x2E079000; 
  fontdata['u'][0]=0x7AEC9F00; 
  fontdata['v'][0]=0x7E900000;
  fontdata['w'][0]=0x7DBF9000; 
  fontdata['x'][0]=0x7F09D000; 
  fontdata['y'][0]=0x7B9D0000; 
  fontdata['z'][0]=0x79DF0000; 
  fontdata['{'][0]=0x3257BEF0; 
  fontdata['|'][0]=0x2E000000;
  fontdata['}'][0]=0x1259BED0; 
  fontdata['~'][0]=0xA7B90000; 
  fontdata[127][0]=0; 		// DEL - wie Leerzeichen
}

void GrafikAusgabe::DrawChar(fontchar letter) const
{
  short j, last;
  unsigned int poswert, poscont;

  poswert = 0xF0000000;
  last = 0;
  for(j=0; j<=15; j++)
  {
	if (j ==  8) poswert = 0xF0000000;
	if (j == 24) poswert = 0x0000000F;
	if ((j <= 7) || (j >= 24)) poscont = ( letter[0] & poswert);
			      else poscont = ( letter[1] & poswert);
	while (poscont > 15) poscont = poscont >> 4;
	if ((last == 0) && (poscont > 0)) 
				glBegin(GL_LINE_STRIP);
	switch ( poscont ) {
		case 1 :   		glVertex2f(     0, 0); break;
		case 2 :		glVertex2f(size/4, 0); break;
		case 3 :		glVertex2f(size/2, 0); break;
		case 4 :		glVertex2f(     0, -size/4); break;
		case 5 :		glVertex2f(size/4, -size/4); break;
		case 6 :		glVertex2f(size/2, -size/4); break;
		case 7 :		glVertex2f(     0, -size/2); break;
		case 8 :		glVertex2f(size/4, -size/2); break;
		case 9 : 		glVertex2f(size/2, -size/2); break;
		case 0xA :	  	glVertex2f(     0, -3*size/4); break;
		case 0xB :	   	glVertex2f(size/4, -3*size/4); break;
		case 0xC :		glVertex2f(size/2, -3*size/4); break;
		case 0xD : 	   	glVertex2f(     0, -size); break;
		case 0xE :		glVertex2f(size/4, -size); break;
		case 0xF :		glVertex2f(size/2, -size); break;
	  }
	if ((last > 0) && (poscont == 0)) glEnd();
	if (j == 15) 
	  {	
		if (poscont > 0) glEnd(); 
		//glTranslatef(size/32, size/32, 0);
		//if (poscont > 0) glBegin(GL_LINE_STRIP);
	  }
	last = poscont;
	if (j <= 15) poswert = poswert >> 4;
		else poswert = poswert << 4;
	if (j == 31) 
	  {		
		if (poscont > 0) glEnd();
		glTranslatef(-size/32, -size/32, 0);
	  };
  };
}

void GrafikAusgabe::DrawGlutChar(char c) const
{
  short		condition = 0;
  const float  	stheight = 130.0f;	// Standard-Height
  float 	charwidth;

  charwidth = glutStrokeWidth(GLUT_STROKE_ROMAN, c);
  glTranslatef(-(charwidth/stheight)/2.0, -size, 0);
  glTranslatef(3.0*size/8.0, 0, 0);
  glScalef(size/stheight, size/stheight, size/stheight);
  condition = glIsEnabled(GL_LIGHTING);
  glDisable(GL_LIGHTING);
  glutStrokeCharacter(GLUT_STROKE_ROMAN, c);
  if (condition) glEnable(GL_LIGHTING);
  glScalef(stheight/size, stheight/size, stheight/size);
  glTranslatef(-(charwidth/stheight)/2, size, 0);
  glTranslatef(-3.0*size/8.0, 0, 0);
};

int  GrafikAusgabe::Compare(const GrafikAusgabe& rightga) const
{
  int back = 0, textcomp = 0;

  if ( wert <  rightga.wert ) back = -1;
  if ( wert >  rightga.wert ) back =  1;
  if ( wert == rightga.wert ) back =  0;
  if (alpha)
    {
	textcomp = strcmp(text, rightga.text);
	if ( textcomp <  0 ) back = -1;
	if ( textcomp >  0 ) back =  1;
	if ( textcomp == 0 ) back = back;
    };

  return back;
};

const GrafikAusgabe& GrafikAusgabe::operator = (const GrafikAusgabe& newga)
{
   if (this != &newga)
     {
	newga.GetText( const_cast<char*>(text), &alpha);
	size = newga.GetSize();
	wert = newga.GetWert();
	newga.GetPos(&posx, &posy, &posz);
	newga.GetColor(&red, &green, &blue);
     };
   return *this;
};

void GrafikAusgabe::DrawAusgabe()
{
  GLfloat ausg_amb[] = {red, green, blue};

  short ziffer[10], i, j, last, maxstellen, aktxpos=1;
  unsigned int  poswert, zwwert;
  fontchar letter;
  

  glPushMatrix();
  glTranslatef(posx, posy, posz);

  poswert = 1000 * 1000 * 1000;		// max. 1 Milliarde als Zahl
  zwwert = wert; 
  maxstellen = 10;
  if (!alpha)
    {
      last = 0;
      for(i=9; i>=0; i--)
       {
	if ( !(ziffer[i] = (zwwert / poswert)) && (last == 0) ) maxstellen--;
	  else last = 1;
	zwwert = zwwert % poswert;
	poswert = poswert / 10;
       } 
      if (maxstellen == 0) maxstellen = 1;
    }
   else maxstellen = strlen(text);

  glColor3f(red, green, blue);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ausg_amb);
  for(i=maxstellen-1; i>=0; i--)
    { 
	if (alpha) 
	  {
		if (text) zwwert = text[maxstellen-i-1];
		  else zwwert = 32;
		if ((zwwert < 0) || (zwwert > 127)) 
		  zwwert = 32; //nicht darstellbare Zeichen auf ' '
		letter = fontdata[ zwwert ];
	  }
	 else letter = numberfont[ ziffer[i] ];
	
	if (alpha && USE_GLUT_FONT)	DrawGlutChar(zwwert);
	  else				DrawChar( letter );

	glTranslatef(3.0f*size/4.0f, 0, 0);
	aktxpos++;	

	if (( zwwert == '\t' ) && ( alpha))	
					// Tabulator, nchste 8er Position
	  {
		j = 8-((aktxpos-2) % 8)-1;
		glTranslatef( (j)*3*size/4.0f, 0, 0);
		aktxpos = aktxpos + j;
	  }
	if (( zwwert == '\n' ) && ( alpha))	
					// Newline, nchste Zeile, auf Anfang
	  {
		glTranslatef( -(aktxpos-1)*3*size/4.0f, -5*size/4.0f, 0);
		aktxpos = 1;
	  };
	if (( zwwert == '\r' ) && ( alpha))	
					// Return, nchste Zeile, auf Anfang
	  {
		glTranslatef( -(aktxpos-1)*3*size/4.0f, -5*size/4.0f, 0);
		aktxpos = 1;
	  };
    }
  glPopMatrix();
}
       	

