/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <strstream>

#include "../include/highsboard.h"
#include "../include/konstanten.h"
#include "../include/language.h"

//--- class Fileviewer ------------------------------------------------------

Fileviewer::Fileviewer()
{
   int i;

   imax = 0;
   filepagelist = NULL;
   for(i = 0; i < linesize; i++) 
   {
	headstring[i] = '\0';
	footstr1[i] = '\0';
	footstr2[i] = '\0';
   }
   viewfile = "";
   filepagelist = NULL; pagenow = NULL;
   for(i=0; i<pagesize+3; i++) 
	screen[i] = NULL;
};

Fileviewer::Fileviewer(char* headfootstrings[], const char *new_viewfile) 
{
   int i;

   imax = 0;
   filepagelist = NULL;
   for(i = 0; i < linesize; i++) 
   {
	headstring[i] = '\0';
	footstr1[i] = '\0';
	footstr2[i] = '\0';
   }
   if (headfootstrings[0]) 
	strncpy(headstring, headfootstrings[0], linesize);
   headstring[linesize-1] = '\0';
   if (headfootstrings[1]) 
	strncpy(footstr1, headfootstrings[1], linesize);
   footstr1[linesize-1] = '\0';
   if (headfootstrings[2]) 
	strncpy(footstr2, headfootstrings[2], linesize - 3);
   footstr2[linesize-1] = '\0';
   
   viewfile = new_viewfile;
   for(i=0; i<pagesize+3; i++) 
     {
	screen[i] = new GrafikAusgabe(-16, 2*(8 - i), 
				      -10.0f, 0);
	screen[i]->SetSize(1.0f);
	screen[i]->SetColor(C_BOARD[0], C_BOARD[1], C_BOARD[2]);
	screen[i]->SetText("");
     };

   Readfile();
   pagenow = filepagelist;
   LoadPage();
};

   
Fileviewer::~Fileviewer()
{
   int i;

   delete filepagelist;
   for(i = 0; i <  12; i++) delete screen[i];
};

void Fileviewer::Scroll(int numofpages)
{
   Doublelist *following;

   following = NULL;
   if (numofpages < 0)
     while (numofpages < 0)
       {
		if (pagenow) pagenow->GetPrev(following);
		if (following) pagenow = following;
		  else	       numofpages = -1;			// break loop
		numofpages++;
       };
   if (numofpages > 0)
     while (numofpages > 0)
       {
		if (pagenow) pagenow->GetNext(following);
		if (following) pagenow = following;
		  else	       numofpages = +1;			// break loop
		numofpages--;
       };
   LoadPage();
};

void Fileviewer::CloseSublist(Objpointerlist<string>* &pagebegin, 
				  Objpointerlist<string>* &pageend,
				  Doublelist* &lastpage)
{
   if (filepagelist) 
  	filepagelist->InsLast(lastpage, pagebegin, pageend);
     else {
		filepagelist = new Doublelist(pagebegin, pageend);
		lastpage = filepagelist;
	  };
   pagebegin = NULL; pageend = NULL;
};

void Fileviewer::Readfile()
{
   int				i = 0, j = 0;
   ifstream 			*filestream;
   char 			zeile[linesize], error = 0;
   string 			*thisline;
   Objpointerlist<string> 	*pagebegin, *pageend;
   Objpointerlist<string>	*filetextlist, *filetlistlast;
   Doublelist 			*lastpage;

   pagebegin = NULL; pageend = NULL; filetextlist = NULL; filetlistlast = NULL;
   lastpage = NULL;
   if (viewfile) filestream = new ifstream(viewfile, ios::in);
     else	 filestream = 0;
   if (filestream) 
     {
	i = 0;
	if (filestream->good())
          while ( filestream->good() )
           {
		for(j = 0; j < linesize; j++) zeile[j] = 0;
		filestream->getline(zeile, linesize - 1);
		if ( ! filestream->eof())
			filestream->clear();
		  else continue;
		thisline = new string(zeile); 
		if (i == 0) 
		  { 
			filetextlist = new Objpointerlist<string>(thisline);
			filetlistlast = filetextlist;
		  }
		  else filetextlist->InsLast(filetlistlast, thisline);
		if ((i % pagesize) == 0) 
		  pagebegin = filetlistlast;
		if ((i % pagesize) == pagesize - 1)
		  {
			pageend = filetlistlast;
			CloseSublist(pagebegin, pageend, lastpage);
		  };
		i = i + 1;
	  	imax = i;
           }
	else error = 1; 	// filestream != good
	//if ( filestream->good() ) cerr << "good" << endl;
	//if ( filestream->fail() ) cerr << "fail" << endl;
	//if ( filestream->eof() ) cerr << "eof" << endl;
	//if ( filestream->bad() ) cerr << "bad" << endl; 
	if (pagebegin)		// save rest in list
	  CloseSublist(pagebegin, pageend, lastpage);
	filestream->close();
	if ( filestream->bad() ) error += 2;
     }
   else error = 1;
   if ( ! viewfile) error = 0;
     else delete filestream;
   if ((error & 1) == 1)
	cerr << S_FILEVIEWER_ERROR_1 << viewfile << "." << endl;
   if ((error & 2) == 2)
	cerr << S_FILEVIEWER_ERROR_2 << viewfile << "." << endl;
};

void Fileviewer::LoadPage()
{
   int 				i = 0, j = 0;
   static char			buffer[pagesize+1][linesize];
   string 			part, *zeile;
   static string		headl;
   strstream 			headline;
   Simplelist 			*begin = NULL;
   Objpointerlist<string> 	*pagebegin = NULL, *aidnext = NULL;

   for(i = 0; i < pagesize; i++)
     for(j = 0; j < linesize; j++)
	buffer[i][j] = 0;
   i = 0;
   if (pagenow) pagenow->GetValue(i);
     else       i = 0;
   headline << headstring << (i * pagesize + 1) << "-"
		<< ((i + 1) * pagesize); 
   headl = "";
   for(i = 0; i <=2; i++) 
     { 
	headline >> part;
	headl += part + " "; 
     };
   headl[linesize-1] = '\0';
   strncpy(buffer[0], headl.data(), headl.length());
   screen[0]->SetText(buffer[0]);
   i = 0;
   if (pagenow) pagenow->GetBegin(begin);
   if (begin) pagebegin = (Objpointerlist<string> *) begin;
   while (i < pagesize)
     {
	if (pagebegin) 
	  {
		pagebegin->GetText(zeile);		// Read line
		strncpy(buffer[i + 1], zeile->data(), linesize - 2);
		if (zeile->length() > 0)
		   screen[i+1]->SetText(buffer[i+1]);	// insert in showtable
		 else screen[i+1]->SetText("");
		pagebegin->GetNext(aidnext);		// go to next
		pagebegin = aidnext;
	  }
	  else screen[i+1]->SetText("");
	i = i + 1;
     }
   screen[pagesize+1]->SetText(footstr1);
   screen[pagesize+2]->SetText(footstr2);
};

void Fileviewer::DrawFile()
{
   int i;

   for(i = 0; i < pagesize+3; i++)
	if (screen[i]) screen[i]->DrawAusgabe();
};

//--- class Highsboard -------------------------------------------------------

Highscoreboard::Highscoreboard(char* headfootstrings[], 
			       const char *highscorefile)
                       : Fileviewer(headfootstrings, highscorefile)
{
   int  i;
   char dummy[linesize];

   for(i = 0; i < linesize; i++) pointsstring[i] = '\0';
   if (headfootstrings[2]) 
	strncpy(pointsstring, headfootstrings[2], linesize - 2);
     else pointsstring[0] = '\0';
   pointsstring[linesize - 3] = '\0';
   pointsstring[linesize - 2] = '\0';
   pointsstring[linesize - 1] = '\0';
   if (headfootstrings[2])
	sscanf(pointsstring, "%d %s", &points, dummy);
    else points = 0;
   highsfile = highscorefile;
};

   
Highscoreboard::~Highscoreboard()
{
   Writefile();
};

void Highscoreboard::GetListLine(Objpointerlist<string>* &scorenow, 
				 string* &zeile)
{
   Objpointerlist<string> *aidnext = NULL;

   zeile = NULL;
   if (scorenow)
     {
	scorenow->GetText(zeile);			// Read line
	scorenow->GetNext(aidnext);			// Go to next
	scorenow = aidnext;
     };
};

void Highscoreboard::Writefile()
{
   int				i=0, highpunkte;
   char				buffer[linesize];
   ofstream 			*filestream;
   string 			*zeile;
   Simplelist 			*begin = NULL;
   Objpointerlist<string> 	*scorenow = NULL;

   for(i = 0; i < linesize; i++) buffer[i] = 0;
   i=0;
   filestream = new ofstream(highsfile, ios::out);
   if (filestream->good())
     {
	if (filepagelist) filepagelist->GetBegin(begin);	// first line
	if (begin) scorenow = (Objpointerlist<string>*) begin;
	GetListLine(scorenow, zeile);
	if (imax > 0) highpunkte = strtol(zeile->data(), (char **)NULL, 10);
	  else highpunkte = -2;
   	i=0;
   	while ( (highpunkte > points) && (i++ < imax) )
     	  {
		strncpy(buffer, zeile->data(), linesize - 2);
		*filestream << buffer << endl;
		GetListLine(scorenow, zeile);
		if (zeile) highpunkte = atoi(zeile->data());
		  else	   highpunkte = -2; 	
     	  };
   	*filestream << pointsstring;
   	*filestream << "\r" << endl;		//Dos-Kompatibel
   	while (i++ < imax) 
     	  {
		strncpy(buffer, zeile->data(), linesize - 2);
		*filestream << buffer << endl;
		GetListLine(scorenow, zeile);
		if (!zeile) i = imax;
     	  }	
   	filestream->close();
	if ( !filestream->good() ) cerr << S_HIGHSBOARD_ERROR << endl;
     }
    else 
     { 
	cout << pointsstring;
	cout << "\r" << endl;			//Dos-Kompatibel
     }
   delete filestream;  
};

