/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <string.h>
#include <iostream>

#include "../include/listen.h"

//*** class Simplelist *******************************************************

Simplelist::~Simplelist()			// Delete all the list
{
   Simplelist *aid, *aidnext, *aidnextnext;

   while (next)					// while exist next
     {
	aid = this; aidnext = next;
   	while (aidnext)				// find last
     	  {
		aidnext->GetNext(aidnextnext);	
		if (!aidnextnext) aid->SetNext(NULL);
						// take out of list
		aid = aidnext;
		aidnext = aidnextnext;
     	  };
   	delete aid;				// and delete it
     };
};

void Simplelist::GetNext(Simplelist* &thisnext) const
{
   thisnext = next;
};

void Simplelist::SetNext(Simplelist* const newnext)
{
   next = newnext;
};

void Simplelist::GetValue(Valuetype &thisvalue) const
{
   thisvalue = value;
};

void Simplelist::SetValue(const Valuetype newvalue)
{
   value = newvalue;
};

void Simplelist::FindValue(const Valuetype newvalue, 
			   Simplelist* &newlist) const
// return one before for DelNext, doesn't find this, 
// also if this->value is newvalue
{
   Valuetype 		thisvalue;
   const Simplelist 	*aid = this; 
   Simplelist 		*aidnext;

   aidnext = next; 
   if (aidnext) aidnext->GetValue(thisvalue); 
   while ( thisvalue != newvalue )
     {
	aid = aidnext;					// Goto Next
	if (!(aidnext)) thisvalue = newvalue;		// If over last break	
	  else          aid->GetNext(aidnext);		// Save Next	
	if (aidnext) aidnext->GetValue(thisvalue);	// which value;
     };
   if (aid != aidnext) newlist = const_cast<Simplelist*> (aid);	// found there
     else	       newlist = NULL;			// found nothing
};

int Simplelist::DelFirst()
{
   int back = 0;
   Simplelist *aidnext;

   aidnext = next;
   if (aidnext)
     {
	aidnext->GetValue(value);		// copy data from next
	aidnext->GetNext(next);
	aidnext->SetNext(NULL);			// take out of list
	delete aidnext;
     }
     else back = 1;  				// use Delete if next is empty;

   return back;
};

int Simplelist::DelNext()
{
   int back = 0;
   Simplelist *aidnext;

   aidnext = next;
   if (aidnext) 
     {
	aidnext->GetNext(next);			// take out of list
   	aidnext->SetNext(NULL);		
   	delete aidnext;				// and delete it
     }
     else back = 1;				// use delete if next is empty

   return back;
};

void Simplelist::InsLast(Simplelist* &last, const Valuetype newvalue)
{
   Simplelist *aid, *aidnext;

   aid = new Simplelist(newvalue);		// Build new with value
   if (last) 
     {
	last->GetNext(aidnext);			// insert in list
	aid->SetNext(aidnext);
	last->SetNext(aid);
     };
   last = aid;					// get new last
};

void Simplelist::InsFirst(const Valuetype newvalue)
{
   Simplelist *aid;

   aid = new Simplelist(value);			// Build new with this value
   aid->SetNext(next);				// Insert in list
   next = aid;
   value = newvalue;				// newvalue here
						// inslast, values exchanged
};

//*** class Doublelist *******************************************************

Doublelist::~Doublelist()
{ 
   if (prev) prev->SetNext(NULL);
   if (next) delete next;
   next = NULL;
   if (subend) subend->SetNext(NULL);	// take out of list
   if (subbegin) delete subbegin;
   subbegin = NULL;
};

void Doublelist::InsLast(Doublelist* &last, 
			 Simplelist* const newbegin, Simplelist* const newend) 
{
   Valuetype newvalue;
   Doublelist *aid, *aidnext;

   aid = new Doublelist(newbegin, newend);	// Build new with newtext
   if (last) 
     {
	last->GetNext(aidnext);			// insert in list
	aid->SetNext(aidnext);
	last->SetNext(aid);
	if (aidnext) aidnext->SetPrev(aid);
	aid->SetPrev(last);
	last->GetValue(newvalue);
	newvalue++;				// use as index
	aid->SetValue(newvalue);
     };
   last = aid;					// get new last
};

void Doublelist::InsFirst(Simplelist* const newbegin, Simplelist* const newend) 
{
   Doublelist *aid, *aidnext;

   aid = new Doublelist(subbegin, subend);	// Build new with this begin, end
   aid->SetNext(next);				// Insert in list
   aidnext = next;
   next = aid;
   if (aidnext) aidnext->SetPrev(aid);
   aid->SetPrev(this);
   subbegin = newbegin;				// newvalues here
   subend = newend;
   aid->SetValue(value);			// this index in new one
   value--;   					// new value for this
						// inslast, values exchanged
};

int  Doublelist::DelFirst() 
{
   int back = 0;
   Doublelist *aidnext, *aidnextnext;

   aidnext = next;
   if (aidnext)
     {
	aidnext->GetValue(value);		// copy data from next
	aidnext->GetBegin(subbegin);
	aidnext->GetEnd(subend);
	aidnext->SetBegin(NULL);		// take out old data
	aidnext->SetEnd(NULL);
	aidnext->GetNext(aidnextnext);		// take out of list	
	if (aidnextnext) 		
	  aidnextnext->SetPrev(this);
	next = aidnextnext;
	aidnext->SetPrev(NULL);
	aidnext->SetNext(NULL);			
	delete aidnext;				// Delete it
     }
     else back = 1;  				// use Delete if next is empty;

   return back;
};

int  Doublelist::DelNext() 
{
   int back = 0;
   Doublelist *aidnext;

   aidnext = next;
   if (aidnext) 
     {
	aidnext->GetNext(next);			// take out of list
	if (next) next->SetPrev(this);
	aidnext->SetPrev(NULL);
	aidnext->SetNext(NULL);
	delete aidnext;				// and delete it
     }
     else back = 1;				// use delete if next is empty

   return back;
};

void Doublelist::FindBegin(Simplelist* const newbegin, 
			   Doublelist* &newlist) const
// return one before for delnext, don't find this subbegin, 
// also if subbegin = newbegin
{
   Simplelist	 	*thisbegin;
   const Doublelist 	*aid = this;
   Doublelist 		*aidnext;

   aid = this; aidnext = next; 
   if (aidnext) aidnext->GetBegin(thisbegin); 
   while ( thisbegin != newbegin )
     {
	aid = aidnext;					// Goto Next
	if (!(aidnext)) thisbegin = newbegin;		// If over last break	
	  else          aid->GetNext(aidnext);		// Save Next	
	if (aidnext) aidnext->GetBegin(thisbegin);	// which begin;
     };
   if (aid != aidnext) newlist = const_cast<Doublelist*> (aid);	// found there
     else	       newlist = NULL;			// found nothing
};

void Doublelist::FindEnd(Simplelist* const newend, 
			 Doublelist* &newlist) const
// return one before for delnext, don't find this subend, 
// also if subend = newend
{
   Simplelist *thisend;
   const Doublelist *aid = this; 
   Doublelist *aidnext;

   aid = this; aidnext = next; 
   if (aidnext) aidnext->GetEnd(thisend); 
   while ( thisend != newend )
     {
	aid = aidnext;					// Goto Next
	if (!(aidnext)) thisend = newend;		// If over last break	
	  else          aid->GetNext(aidnext);		// Save Next	
	if (aidnext) aidnext->GetEnd(thisend);		// which end;
     };
   if (aid != aidnext) newlist = const_cast<Doublelist*> (aid);	// found there
     else	       newlist = NULL;			// found nothing
};

void Doublelist::FindValue(const Valuetype newvalue, 
			   Doublelist* &newlist) const
// return one before for DelNext, doesn't find this, 
// also if this->value is newvalue
{
   Valuetype 		thisvalue;
   const Doublelist 	*aid = this; 
   Doublelist 		*aidnext;

   aidnext = next; 
   if (aidnext) aidnext->GetValue(thisvalue); 
   while ( thisvalue != newvalue )
     {
	aid = aidnext;					// Goto Next
	if (!(aidnext)) thisvalue = newvalue;		// If over last break	
	  else          aid->GetNext(aidnext);		// Save Next	
	if (aidnext) aidnext->GetValue(thisvalue);	// which value;
     };
   if (aid != aidnext) newlist = const_cast<Doublelist*> (aid);	// found there
     else	       newlist = NULL;			// found nothing
};

void Doublelist::GetBegin(Simplelist* &thisbegin) const
{
   thisbegin = subbegin;
};

void Doublelist::GetEnd(Simplelist* &thisend) const
{
   thisend = subend;
};

void Doublelist::SetBegin(Simplelist* const newbegin) 
{
   subbegin = newbegin;
};

void Doublelist::SetEnd(Simplelist* const newend) 
{
   subend = newend;
};

void Doublelist::GetPrev(Doublelist* &thisprev) const
{
   thisprev = prev;
};

void Doublelist::SetPrev(Doublelist* const newprev) 
{
   prev = newprev;
};

void Doublelist::GetNext(Doublelist* &thisnext) const 
{
   thisnext = next;
};

void Doublelist::SetNext(Doublelist* const newnext) 
{
   next = newnext;
};

