/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

// Test program for class listen
// g++ -g -c listen.cpp -o listen.o
// g++ -g -c listencheck.cpp -o listencheck.o
// g++ -o listencheck listen.o listencheck.o

#include <string.h>
#include <string>
#include <strstream>

#include "../include/listen.h"

void showsimplelist(Simplelist *list)
{  
   int value;
   Simplelist *aidnext;

   printf("List: ");
   while (list != NULL)				// while exist a list
     {
	list->GetValue(value);			
	printf("%2d ", value);			// print value
	list->GetNext(aidnext);			// Goto next
	list = aidnext;
     };
   printf("\n");
};

void testsimplelist()
{
   int i;
   Simplelist *head, *last;

   head = new Simplelist(0);
   last = head;
   
   printf("InsLast\n"); 
   for(i=1; i<=10; i++)
     head->InsLast(last, i);
   showsimplelist(head);
   printf("InsFirst\n"); 
   head->InsFirst(i);
   showsimplelist(head);
   printf("Delnext on head\n"); 
   head->DelNext();
   showsimplelist(head);
   printf("Delfirst on head\n"); 
   head->DelFirst();
   showsimplelist(head);

   printf("Delnext on last\n"); 
   last->DelNext();
   showsimplelist(head);
   printf("Delfirst on last\n"); 
   last->DelFirst();
   showsimplelist(head);
   printf("Delete last with FindValue of last\n");
   last->GetValue(i);
   head->FindValue(i, last);
   last->DelNext();
   showsimplelist(head);
   printf("Findvalue+DelNext\n"); 
   head->FindValue(5, last);
   last->DelNext();
   showsimplelist(head);
   printf("Find first value (1) and show empty result\n");
   head->FindValue(1, last);
   showsimplelist(last);
   printf("Find not used value (12) and show empty result\n");
   head->FindValue(12, last);
   showsimplelist(last);
   printf("Delete\n"); 
   delete head;
   showsimplelist(head);
};

void test_constlisten()
{
   int 				value;
   const Simplelist 		csimplelist(1); 

   csimplelist.GetValue(value);
   cout << "Constlist 1 content: " << value << endl;
};

void showstringlist(Objpointerlist<string> *list)
{  
   int value;
   Objpointerlist<string> *aidnext;

   while (list != NULL)				// while exist a list
     {
	list->GetValue(value);			
	cout << " \"" << (*list) << "\" ";	// print content
	cout << value;
	list->GetNext(aidnext);			// Goto next
	list = aidnext;
     };
   cout << endl;
};

void teststringlist()
{
   int 				i;
   char	 			textc[3];
   string 			*text;
   strstream 			stringstr;
   Objpointerlist<string> 	*head, *last;
   
   text = new string;
   sprintf(textc, "%d", 0);
   *text = textc;
   head = new Objpointerlist<string>(text);
   last = head;

   printf("Inslast\n");
   for(i=1; i<=10; i=i+1)
     {
	text = new string;
   	sprintf(textc, "%d", i);
	if (i==0) i=10;
   	*text = textc;
	head->InsLast(last, text);
     };
   showstringlist(head);
   printf("InsFirst\n"); 
   text = new string;
   sprintf(textc, "%d", i);
   *text = textc;
   head->InsFirst(text);
   showstringlist(head);
   printf("Delnext on head and change first string.\n"); 
   stringstr << -1;
   stringstr >> *head;
   head->DelNext();
   showstringlist(head);
   printf("Delfirst on head\n"); 
   head->DelFirst();
   showstringlist(head);

   printf("Delnext on last\n"); 
   last->DelNext();
   showstringlist(head);
   printf("Delfirst on last\n"); 
   last->DelFirst();
   showstringlist(head);
   printf("Delete last with FindText of last\n");
   last->GetText(text);
   head->FindText(*text, last);
   last->DelNext();
   showstringlist(head);
   printf("FindText+DelNext\n"); 
   head->FindText("5", last);
   last->DelNext();
   showstringlist(head);
   printf("Find first Text (\"1\") and show empty result\n");
   head->FindText("1", last);
   showstringlist(last);
   printf("Find not used Text (\"12\") and show empty result\n");
   head->FindText("12", last);
   showstringlist(last);
   printf("Delete\n"); 
   delete head;
   head = NULL;
   showstringlist(head);
};

void showdoublelist(Doublelist *list)
{  
   int value;
   Simplelist *begin;
   Doublelist *aidnext;

   printf("Doublelist: \n");
   while (list)					// while exist a list
     {
	list->GetValue(value);			
	printf(" %2d ", value);
	list->GetBegin(begin);
	showsimplelist(begin);			// print content
	list->GetNext(aidnext);			// Goto next
	list = aidnext;
     };
   printf("\n");
};

void testdoublelist()
{
   int i;
   Doublelist *head, *last;
   Simplelist *begin, *end;
   
   begin = new Simplelist(0);
   end = begin;
   begin->InsLast(end, 1); begin->InsLast(end, 2);
   begin->InsLast(end, 3); begin->InsLast(end, 4);
   head = new Doublelist(begin, end);
   last = head;
   for (i=5; i<=20; i=i+5)
     {
	begin = new Simplelist(i);
	end = begin;
   	begin->InsLast(end, i+1); begin->InsLast(end, i+2);
   	begin->InsLast(end, i+3); begin->InsLast(end, i+4);
   	head->InsLast(last, begin, end);
     };
   showdoublelist(head);
   printf("InsFirst\n");
   begin = new Simplelist(-5);
   end = begin;
   begin->InsLast(end, -4); begin->InsLast(end, -3);
   begin->InsLast(end, -2); begin->InsLast(end, -1);
   head->InsFirst(begin, end);
   showdoublelist(head);
   printf("Delnext on head\n"); 
   head->DelNext();
   showdoublelist(head);
   printf("Delfirst on head\n"); 
   head->DelFirst();
   showdoublelist(head);

   printf("Delnext on last\n"); 
   last->DelNext();
   showdoublelist(head);
   printf("Delfirst on last\n"); 
   last->DelFirst();
   showdoublelist(head);
   printf("Delete last with FindBegin of last\n");
   last->GetBegin(begin);
   head->FindBegin(begin, last);
   last->DelNext();
   showdoublelist(head);
   printf("FindValue+DelNext\n"); 
   head->FindValue(2, last);
   last->DelNext();
   showdoublelist(head);
   printf("Find first Value (1) and show empty result\n");
   head->FindValue(1, last);
   showdoublelist(last);
   printf("Find not used Value (12) and show empty result\n");
   head->FindValue(12, last);
   showdoublelist(last);
   printf("Delete\n"); 
   delete head;
   showdoublelist(head);
};

int main()
{
   testsimplelist();
   printf("---------------------\n");
   test_constlisten();
   printf("---------------------\n");
   teststringlist();
   printf("---------------------\n");
   testdoublelist();
   return 0;
};

