/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "../include/mann.h"
#include "../include/spielraum.h"

//---Klasse Mann----------------------------------

Mann::Mann(float neugroesse, float neubreite, int new_level_num,
	   const Texture* *neutexarray):
	level_num(new_level_num), texarray(neutexarray)
{
  this->richtung = 0;
  this->groesse = neugroesse;
  this->breite = neubreite;
  this->posx = MAXFELD;
  this->posy = MAXFELD;
  this->jump = 0;
  this->schenken = new Geschenke(ZUMGEWINN % 100, ZUMGEWINN/100);
  qmann = gluNewQuadric();
  manndlist = glGenLists(1);
  DrawParts(manndlist);
};

Mann::~Mann()
{
  groesse = 0; breite = 0; posx = 0; posy = 0; jump = 0;
  if (glIsList(manndlist)) glDeleteLists(manndlist, 1);
  delete schenken;
  texarray = NULL;
  gluDeleteQuadric(qmann);
};

void Mann::Fahrstuhl(Spielraum *spraum, int neuebene)
{
  int zufall;

  if (schenken->Fertig() == 1) neuebene = MAXEBENEN-1;
  if (neuebene > 0)				//Sonst kein Fahrstuhl da
    {
	zufall = rand() % CHVERLIEREN;
	if (schenken->Fertig() == 1) neuebene = MAXEBENEN-1;
	  else if (zufall == 0) neuebene = 0;		//Pech gehabt
	if ((neuebene != MAXEBENEN-1) && (neuebene > 0))
		neuebene = spraum->SwitchLevel(neuebene);
	if (neuebene != level_num)
	  {
		this->posx = MAXFELD;
		this->posy = MAXFELD;
	  };
	if (neuebene ==  0) spraum->SwitchEnd(-1);
	if (neuebene == MAXEBENEN-1) 
		spraum->SwitchEnd(schenken->CalcPunkte());
    };
  if (neuebene > 0) level_num = neuebene;
};

void Mann::SetPosView()
{
  float xview, yview, zview;

  xview = sin(richtung*3.1415/180)*MAXFELD*4;
  if (jump == 2) yview = groesse/4 + MAXFELDHOEHE*FELDEINHEIT/4;
    else	 yview = groesse/4;
  zview = cos(richtung*3.1415/180)*MAXFELD*4;
  glLoadIdentity();
  gluLookAt(posx*2-xview/MAXFELD/16, 8*yview, posy*2-zview/MAXFELD/16,
	    posx*2-xview, -10*yview, posy*2-zview, 0, 1, 0);
};

void Mann::AddPos(Spielraum *spraum, int x, int y, int j)
{
  int 		neuposx, neuposy, erg, boolvar, etage;
  Feldpunkte 	*content;

  switch (j)
    {		// Aufladen mit j=1, dann j=0 springen(jump=2), dann aus.
	case 0 : if (jump == 1) jump = 2; else jump = 0; break;
	case 1 : if (jump < 2 ) jump = 1; break;
    };
  neuposx = this->posx + x;
  neuposy = this->posy + y;
  boolvar = (neuposx <= MAXFELD) && (neuposx >= -MAXFELD);
  boolvar = (boolvar) && (neuposy <= MAXFELD) && (neuposy >= -MAXFELD);
  if (boolvar)
    {
	if (jump < 1)
	{
	  erg = spraum->GetContent(neuposx, neuposy, content);
	  if (erg <= 0)		// typlos / huegel 
	    {
		this->posx = neuposx; this->posy = neuposy;
	    };
          if (erg >  0) 	// baum
		*schenken += *content;
	  if (erg <= -1) 	// huegel
	    {
		etage = content->Getkanz() + content->Getganz() * 10;
		this->Fahrstuhl( spraum, etage );
	    };
	  if (content) delete content;
	}
	else { 
		this->posx = neuposx; this->posy = neuposy;
		erg = spraum->GetAbove(neuposx, neuposy, content);
		if (content) 
		  { 
			*schenken += *content;
			delete content;
		  };
	     } // of else
    } // of if(boolvar)

  if (spraum->GetViewMann()) SetPosView(); 
};

void Mann::NeuRichtung(int winkel)
{
  this->richtung = winkel;
}

int Mann::GetRichtung()
{
  return richtung;
};

void Mann::DrawParts(GLuint listdisplay)
{
  float sizeunit, widthunit;
  int detail;

  if (MANNDETAILS) detail = DETAILS;
    else	   detail = 10;
  sizeunit = groesse / 10;
  widthunit = this->breite / 2;
  glNewList(listdisplay, GL_COMPILE);			// until end
  glRotatef(270, 1, 0, 0);

  glTranslatef(-widthunit, widthunit, 0);
  if (texarray[6])
	glBindTexture(GL_TEXTURE_2D, texarray[6]->GetTexNumber() );
  glColor3fv(C_SHOES);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, SHOES_AMB);
  gluCylinder(qmann, widthunit, widthunit, sizeunit, detail, detail/2);		// Schuhe
  glTranslatef(widthunit*2, 0, 0);
  gluCylinder(qmann, widthunit, widthunit, sizeunit, detail, detail/2);
  glBindTexture(GL_TEXTURE_2D, 0);

  glTranslatef(-widthunit, -widthunit, sizeunit);
  if (texarray[5])
	glBindTexture(GL_TEXTURE_2D, texarray[5]->GetTexNumber() );
  glColor3fv(C_BODY);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, BODY_AMB);
  gluCylinder(qmann, widthunit*2, widthunit, 7.5*sizeunit, detail, detail/2);	// Krper
  glBindTexture(GL_TEXTURE_2D, 0);

  glTranslatef(0, 0, 7.5*sizeunit);
  glRotatef( 90, 0, 1, 0);
  glRotatef(145, 0, 0, 1);
  if (texarray[4])
	glBindTexture(GL_TEXTURE_2D, texarray[4]->GetTexNumber() );
  glColor3fv(C_FACE);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, FACE_AMB);
  gluSphere(qmann, 1.2*sizeunit, detail, detail);				// Kopf
  glRotatef(195, 0, 0, 1);
  glRotatef(270, 0, 1, 0);
  glBindTexture(GL_TEXTURE_2D, 0);

  if (MANN_CAP)
    {
	glTranslatef(0, -sizeunit/2, sizeunit/2);
	glRotatef(45, 1, 0, 0);
	if (texarray[3])
		glBindTexture(GL_TEXTURE_2D, texarray[3]->GetTexNumber() );
  	glColor3fv(C_CAP);
  	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, CAP_AMB);
  	gluCylinder(qmann, 1.4*sizeunit, 0, this->groesse/3, 
			detail, detail/2);	// Mtze
  	glRotatef(315, 1, 0, 0);
  	glBindTexture(GL_TEXTURE_2D, 0);
    }

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, MATERIAL_NULL);
  glRotatef(90, 1, 0, 0);
  glEndList();
};

void Mann::DrawMann()
{
  glPushMatrix();
  if (jump == 2) glTranslatef(posx * 2, MAXFELDHOEHE*FELDEINHEIT, posy * 2);
    else glTranslatef(posx * 2, 0, posy * 2);
  glRotatef(richtung, 0, 1, 0);

  glCallList(manndlist);

  glRotatef(360-richtung, 0, 1, 0); 
  if (jump == 1) glTranslatef(0, -(MAXFELDHOEHE-2)*FELDEINHEIT, 0);
  glPopMatrix();
};

void Mann::DrawPunkte()
{
  schenken->DrawGeschenke();
};



