/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <iostream>

//*** class Objpointerlist ***************************************************

template <class Texttype>
Objpointerlist<Texttype>::~Objpointerlist() 
{
   Objpointerlist *aid, *aidnext, *aidnextnext;

   if (textstring) delete textstring;
   while (next)					// while exist next
     {
	aid = this; aidnext = next;
   	while (aidnext)				// find last
     	  {
		aidnext->GetNext(aidnextnext);	
		if (!aidnextnext) aid->SetNext(NULL);
						// take out of list
		aid = aidnext;
		aidnext = aidnextnext;
     	  };
   	delete aid;				// and delete it
     };
};

template <class Texttype>
void Objpointerlist<Texttype>::GetValue(Valuetype &thisvalue) const
{
   thisvalue = value;
};

template <class Texttype>
void Objpointerlist<Texttype>::SetValue(const Valuetype newvalue) 
{
   value = newvalue;
};

template <class Texttype>
void Objpointerlist<Texttype>::GetText(Texttype* &thistext) const
{
   thistext = textstring;
};

template <class Texttype>
void Objpointerlist<Texttype>::SetText(Texttype* const newtext) 
{
   textstring = newtext;
};

template <class Texttype>
istream& operator >> (istream& instream, 
		      Objpointerlist<Texttype>& thisobjl)
{
   Texttype *text;

   text = new Texttype();
   instream >> *text;
   thisobjl.SetText(text);
   return instream;
};

template <class Texttype>
ostream& operator << (ostream& outstream, 
		      const Objpointerlist<Texttype>& thisobjl)
{
   Texttype *text;

   thisobjl.GetText(text);
   outstream << *text;
   return outstream;
};
 
template <class Texttype>
void Objpointerlist<Texttype>::GetNext(Objpointerlist* &thisnext) const
{
   thisnext = next;
};

template <class Texttype>
void Objpointerlist<Texttype>::SetNext(Objpointerlist* const newnext)
{
   next = newnext;
};

template <class Texttype>
void Objpointerlist<Texttype>::InsLast(Objpointerlist* &last, 
					Texttype* const newtext) 
{
   Valuetype newvalue;
   Objpointerlist *aid, *aidnext;

   aid = new Objpointerlist(newtext);		// Build new with newtext
   if (last) 
     {
	last->GetNext(aidnext);			// insert in list
	aid->SetNext(aidnext);
	last->SetNext(aid);
	last->GetValue(newvalue);
	newvalue++;				// use as index
	aid->SetValue(newvalue);
     };
   last = aid;					// get new last
};

template <class Texttype>
void Objpointerlist<Texttype>::InsFirst(Texttype* const newtext) 
{
   Objpointerlist *aid;

   aid = new Objpointerlist(textstring);	// Build new with this textstring
   aid->SetNext(next);				// Insert in list
   next = aid;
   textstring = newtext;			// newvalue here
   aid->SetValue(value);			// this index in new one
   value--;   					// new value for this
						// inslast, values exchanged
};

template <class Texttype>
int  Objpointerlist<Texttype>::DelFirst() 
{
   int back = 0;
   Texttype *nexttext;
   Objpointerlist *aidnext;

   aidnext = next;
   if (aidnext)
     {
	aidnext->GetValue(value);		// copy data from next
	aidnext->GetText(nexttext);
	aidnext->SetText(textstring);		// exchange Strings
	textstring = nexttext;
	aidnext->GetNext(next);
	aidnext->SetNext(NULL);			// take out of list
	delete aidnext;
     }
     else back = 1;  				// use Delete if next is empty;

   return back;
};

template <class Texttype>
int  Objpointerlist<Texttype>::DelNext() 
{
   int back = 0;
   Objpointerlist *aidnext;

   aidnext = next;
   if (aidnext) 
     {
	aidnext->GetNext(next);			// take out of list
   	aidnext->SetNext(NULL);		
   	delete aidnext;				// and delete it
     }
     else back = 1;				// use delete if next is empty

   return back;
};

template <class Texttype>
void Objpointerlist<Texttype>::FindText(const Texttype newtext, 
				    	Objpointerlist* &newlist) const
// return one before for DelNext, doesn't find this, 
// also if this->text is newtext
{
   Texttype* 		thistext=NULL;
   const Objpointerlist *aid = this;
   Objpointerlist 	*aidnext;

   aidnext = next; 
   if (aidnext) aidnext->GetText(thistext); 
   while ( *thistext != newtext )
     {
	aid = aidnext;					// Goto Next
	if (!(aidnext)) *thistext = newtext;		// If over last break	
	  else          aid->GetNext(aidnext);		// Save Next	
	if (aidnext) aidnext->GetText(thistext);	// which text;
     };
   if (aid != aidnext) newlist = const_cast<Objpointerlist*> (aid);
							// found there
     else	       newlist = NULL;			// found nothing
};

template <class Texttype>
void Objpointerlist<Texttype>::FindValue(const Valuetype newvalue, 
				     	 Objpointerlist* &newlist) const
// return one before for DelNext, doesn't find this, 
// also if this->value is newvalue
{
   Valuetype 		thisvalue;
   const Objpointerlist *aid = this;
   Objpointerlist 	*aidnext;

   aidnext = next; 
   if (aidnext) aidnext->GetValue(thisvalue); 
   while ( thisvalue != newvalue )
     {
	aid = aidnext;					// Goto Next
	if (!(aidnext)) thisvalue = newvalue;		// If over last break	
	  else          aid->GetNext(aidnext);		// Save Next	
	if (aidnext) aidnext->GetValue(thisvalue);	// which value;
     };
   if (aid != aidnext) newlist = const_cast<Objpointerlist*> (aid);
							// found there
     else	       newlist = NULL;			// found nothing
};

