/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler 
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify  
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public 
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root 
directory of this package.
*/
// Joystick nicht getestet !
//
//***Vereinbarungen****************************
//---Includes, Konstanten----------------------

#include <GL/glut.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <time.h>

#include "../include/spielraum.h"
#include "../include/mann.h"
#include "../include/konstanten.h"
#include "../include/highsboard.h"
#include "../include/language.h"

//---Typen- und Klassendefiniton----------------
	
typedef enum{angle, trans, reshape} keymodifmodes;
typedef enum{play, anglexy, anglexz, angleyz, transxy, 
		transxz, transyz, scroll} mousemodifmodes;
typedef enum{mgame, mfileview, mhighscore} program_modes;

//--- globale Variablen -----------------------

int		aktangle[3], savedangle[3];
float		akttrans[3], savedtrans[3];
int 		aktwidth, aktheight, menu_id;
keymodifmodes 	modifmodekey;
mousemodifmodes modifmodemouse;
program_modes	prog_mode;
int		joystickx, joysticky, mousex, mousey;
char		*highscorefile=0, *full_name=0;

Spielraum	*spraum;
Mann		*weihmann[MANN_NUMBER], *weihmann_id;		
Highscoreboard  *highscore;
Fileviewer	*fileview = NULL;

//---Funktions-Deklarationen ----------------------------

void RebuildMenu();
void DoReshape(int, int);
void DoRotate(char, int);
void DoTranslation(char, float);

//***Funktionen, OpenGL-Grundlagen***********************

void initVars()
{
  short i;

  for(i = 0; i <= 2; i++) 		// i = X, Y, Z
    { aktangle[i] = 0; akttrans[i] = 0; };
  modifmodekey = angle; modifmodemouse = play; prog_mode = mgame;
  aktwidth = ANFWIDTH; aktheight = ANFHEIGHT;
};

void CreateFigure()
{
   int i = 0;

   while( i < MANN_NUMBER )  
	if (weihmann[i])  	i++;		// Find unused
	  else 			break;
   if ( i < MANN_NUMBER )
     {
	weihmann[i] = new Mann(3.0, 0.6, spraum->GetLevelNum(), 
				spraum->GetTexarray());
	weihmann_id = weihmann[i];
     };
};

void SwitchFigure()
{
   int i = 0, hislevel, back;

   while( weihmann[i] != weihmann_id )
     i = (i + 1) % MANN_NUMBER;			//Find current
   i = (i + 1) % MANN_NUMBER;			//Goto next
   if (weihmann[i] == NULL) i = 0;		//if not used
   weihmann_id = weihmann[i];
   hislevel = weihmann_id->GetLevelNum();
   if ( spraum->GetLevelNum() != hislevel ) 
	back = spraum->SwitchLevel( hislevel );
};

void initVarsTwo()
{
  int i;

  spraum = new Spielraum();
  for(i = 0; i < MANN_NUMBER; i++) weihmann[i] = NULL;
  CreateFigure();
};

void InitLight01()
{
   GLfloat pos_light0[] = {0.0, MAXFELDHOEHE*FELDEINHEIT*2, MAXFELD*5, 1.0};
   GLfloat pos_light1[] = {0.0, MAXFELDHOEHE*FELDEINHEIT*2, -MAXFELD*5, 1.0};
 
   glEnable(GL_LIGHT0);
   glLightfv(GL_LIGHT0, GL_POSITION, pos_light0);
   glLightfv(GL_LIGHT0, GL_AMBIENT, PLIGHT_AMB);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, PLIGHT_DIF);
   glLightfv(GL_LIGHT0, GL_SPECULAR, PLIGHT_SPEC);

   glEnable(GL_LIGHT1);
   glLightfv(GL_LIGHT1, GL_POSITION, pos_light1);
   glLightfv(GL_LIGHT1, GL_AMBIENT, PLIGHT_AMB);
   glLightfv(GL_LIGHT1, GL_DIFFUSE, PLIGHT_DIF);
   glLightfv(GL_LIGHT1, GL_SPECULAR, PLIGHT_SPEC);
};

//---Ende + Darstellungsfunktion, greift auf die Klassen zu-------

void endeproc()
{
   int i;

   delete highscore;
   weihmann_id = NULL;
   for(i = 0; i < MANN_NUMBER; i++)
     if (weihmann[i]) delete weihmann[i];
   delete spraum;

   glutDestroyMenu(menu_id);
   exit(0);
}

void InitShowFileview(char* headfootstrings[])
{
   const short	bufsize = 250;
   int i;

   headfootstrings[0] = new char[bufsize];
   headfootstrings[1] = new char[bufsize];
   headfootstrings[2] = new char[bufsize];
   for(i = 0; i < bufsize; i++) 
     {
	headfootstrings[0][i] = '\0';
	headfootstrings[1][i] = '\0';
	headfootstrings[2][i] = '\0';
     };
   glutSetMenu(menu_id);
   glutChangeToMenuEntry(1, S_MENU_ENTRY_ID_80, 80);
   modifmodemouse = scroll;
   if (prog_mode == mgame)
   {
   	glutRemoveMenuItem(5);
   	glutRemoveMenuItem(4);
   };
   for(i = 0; i < 3; i++)
   {
	savedangle[i] = aktangle[i];
	savedtrans[i] = akttrans[i];
   };
   DoTranslation('i', 0); 
   DoRotate('i', 0);
   prog_mode = mfileview;
   DoReshape(aktwidth, aktheight);
};

void SwitchToGame()
{
   int i;

   if (spraum->GetEnd() == 0) 	prog_mode = mgame;
     else			prog_mode = mhighscore;
   if (fileview) delete fileview;
   fileview = NULL;
   if (spraum->GetEnd() == 0) RebuildMenu();
   DoReshape(aktwidth, aktheight);

   for(i = 0; i < 3; i++)
   {
	aktangle[i] = savedangle[i];
	akttrans[i] = savedtrans[i];
   };
};

void ShowFile(const char filename[])
{
   const short	bufsize = 250;

   int		i;
   char*	headfootstrings[3];

   if (fileview) SwitchToGame();
   InitShowFileview(headfootstrings);
   prog_mode = mfileview;
   snprintf(headfootstrings[0], bufsize - 1, S_SHOWFILE_HEAD);
   snprintf(headfootstrings[1], bufsize - 1, S_SHOWFILE_FOOT);
   strncpy(headfootstrings[2], filename, bufsize - 1);

   fileview = new Fileviewer(headfootstrings, filename);

   for(i = 0; i <= 2; i++) delete headfootstrings[i];
};

void InitHighscore()
{
   const short	bufsize = 250;

   struct timeb	tp;
   char		zeit[24], pointsstring[bufsize];
   char*	headfootstrings[3];
   int		i;

   InitShowFileview(headfootstrings);
   prog_mode = mhighscore;
   for(i = 0; i < bufsize; i++) pointsstring[i] = ' ';
   snprintf(headfootstrings[0], bufsize-1, S_HIGHSCORE_HEAD);
   snprintf(headfootstrings[1], bufsize-1, S_HIGHSCORE_FOOT);
   sprintf(pointsstring, "%d ", spraum->GetPoints() );
   i = ftime(&tp);
   strncpy(zeit, ctime(&tp.time), strlen(ctime(&tp.time)) + 1 );
   strncat(pointsstring, zeit, strlen(zeit) - 1);
   strcat(pointsstring, " ");
   if (full_name) strncat(pointsstring, full_name,
                          bufsize - strlen(pointsstring) - 2 );
   strncpy(headfootstrings[2], pointsstring, bufsize);

   highscore = new Highscoreboard(headfootstrings, highscorefile);

   for(i = 0; i <= 2; i++) delete headfootstrings[i];
};

void DrawGameScreen()
{
   int i, currentlevel;

   InitLight01();
   glViewport(0, aktheight*4/5, aktwidth, aktheight);
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   gluPerspective(30, 1, 1, 15*4);
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity();
   gluLookAt(0, 0, 4*3, 0, 0, 0, 0, 1, 0);

   spraum->DrawLevelNum();
   weihmann_id->DrawPunkte();

   glViewport(0, 0, aktwidth, aktheight*4/5);
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();

   spraum->DrawSpielraum();
   currentlevel = spraum->GetLevelNum();
   for(i = 0; i < MANN_NUMBER; i++)
     if (weihmann[i]) 
	if (weihmann[i]->GetLevelNum() == currentlevel)
		weihmann[i]->DrawMann();
};

void display(void)
{  
  glEnable(GL_DEPTH_TEST);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glPushMatrix();		
  if ( (spraum->GetEnd() == 1) && (!highscore) ) InitHighscore();
  if ( fileview || highscore) DoReshape(aktwidth, aktheight);
  glTranslatef(akttrans[0]/5, akttrans[1]/5, akttrans[2]/5);
	// X = 0, Y = 1, Z = 2
  glRotatef(aktangle[0], 1, 0, 0);
  glRotatef(aktangle[1], 0, 1, 0);
  glRotatef(aktangle[2], 0, 0, 1);

  if (prog_mode != mhighscore)
    {
	if (prog_mode == mfileview) fileview->DrawFile();
	  else DrawGameScreen();
    }
  else 
    {
	if (spraum->GetEnd() == 1) highscore->DrawFile();
	if (spraum->GetEnd() == 2) endeproc();
    };
  glFlush();
  glPopMatrix();
  glutSwapBuffers();
}

//---Einstellungen fr Darstellung-----------------------------

void DoReshape(int width, int height)
{
   short condition = 0;
   int 	minsize, maxsize;

   aktwidth = width; aktheight = height;
   if (width >= height) { minsize = height; maxsize = width; }
     else { minsize = width; maxsize = height; };
   
   if (prog_mode == mgame)
   {
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0*maxsize/minsize, 1, 1, 15*MAXFELD);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	if (spraum) condition = (!spraum->GetViewMann() );
	if ((condition) || (!spraum))
		gluLookAt(GLOBAL_VIEW[0], GLOBAL_VIEW[1], GLOBAL_VIEW[2], 
		  	  GLOBAL_VIEW[3], GLOBAL_VIEW[4], GLOBAL_VIEW[5],
			  GLOBAL_VIEW[6], GLOBAL_VIEW[7], GLOBAL_VIEW[8]);
	if ((spraum) && (!condition)) weihmann_id->AddPos(spraum, 0, 0, 0);
   }
   else
   {
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0*maxsize/minsize, 1, 1, 15*4);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0, 4, 18, 0, 4, 0, 0, 1, 0);
   };
};

void init (void)
{  
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

   DoReshape(aktwidth, aktheight);

   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LIGHT_AMB);
   glEnable(GL_LIGHTING);
   glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MATERIAL_NULL);
   glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, MATERIAL_NULL);
   glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, MATERIAL_NULL);
}

void DoRotate(char axis, int addvalue)
//axis: 'x', 'y', 'z'; 'i' for init (initvalue = addvalue)
{
   int i;

   switch (axis) {
     	case 'x' : aktangle[0] = (360 + aktangle[0] + addvalue) % 360; break;
     	case 'y' : aktangle[1] = (360 + aktangle[1] + addvalue) % 360; break;
     	case 'z' : aktangle[2] = (360 + aktangle[2] + addvalue) % 360; break;
	case 'i' : for(i=0; i<=2; i++) aktangle[i] = addvalue; break;
     };
};

void DoTranslation(char axis, float addvalue)
//axis: 'x', 'y', 'z'; 'i' for init (initvalue = addvalue)
{
   int i;

   switch (axis) {
	case 'x' : akttrans[0] = (akttrans[0] + addvalue) ; break;
	case 'y' : akttrans[1] = (akttrans[1] + addvalue) ; break;
	case 'z' : akttrans[2] = (akttrans[2] + addvalue) ; break;
	case 'i' : for(i=0; i<=2; i++) akttrans[i] = addvalue; break;
     };
};

//---Men-Verarbeitung---------------------------------------------------

static void Menufunc(int entry)
{
   switch (entry) {
	case  10 : modifmodemouse = play;		break;
	case 210 : modifmodemouse = anglexy;		break;
	case 220 : modifmodemouse = anglexz;		break;
	case 230 : modifmodemouse = angleyz;		break;
	case 240 : DoRotate('i', 0);			break;
	case 310 : modifmodemouse = transxy;		break;
	case 320 : modifmodemouse = transxz;		break;
	case 330 : modifmodemouse = transyz;		break;
	case 340 : DoTranslation('i', 0);		break;
	case  40 : weihmann_id->TriggerPunktAnz();	break;
	case 510 : spraum->TriggerViewMann();		break;
	case 520 : CreateFigure();			break;
	case 530 : SwitchFigure();			break;
	case 610 : ShowFile(S_NAME_HELPFILE);		break;
	case 620 : ShowFile("gpl.txt");			break;
	case  70 : if (prog_mode == mfileview)
			SwitchToGame();
		   else spraum->SwitchEnd(0);		break;
	case  80 : modifmodemouse = scroll;		break;
     };
   glutPostRedisplay();
};

void BuildSubMenus(int &id1, int &id2, int &id3, int &id4)
{
   id1 = glutCreateMenu(Menufunc);
   glutAddMenuEntry(S_MENU_ENTRY_ID_210, 210);
   glutAddMenuEntry(S_MENU_ENTRY_ID_220, 220);
   glutAddMenuEntry(S_MENU_ENTRY_ID_230, 230);
   glutAddMenuEntry(S_MENU_ENTRY_ID_240, 240);

   id2 = glutCreateMenu(Menufunc);
   glutAddMenuEntry(S_MENU_ENTRY_ID_310, 310);
   glutAddMenuEntry(S_MENU_ENTRY_ID_320, 320);
   glutAddMenuEntry(S_MENU_ENTRY_ID_330, 330);
   glutAddMenuEntry(S_MENU_ENTRY_ID_340, 340);

   id3 = glutCreateMenu(Menufunc);
   glutAddMenuEntry(S_MENU_ENTRY_ID_510, 510);
   glutAddMenuEntry(S_MENU_ENTRY_ID_520, 520);
   glutAddMenuEntry(S_MENU_ENTRY_ID_530, 530);

   id4 = glutCreateMenu(Menufunc);
   glutAddMenuEntry(S_MENU_ENTRY_ID_610, 610);
   glutAddMenuEntry(S_MENU_ENTRY_ID_620, 620);

};

void InitMenu()
{
   int sub_menu_rotate_id, sub_menu_translate_id;
   int sub_menu_program_id, sub_menu_mann_id;

   menu_id = glutCreateMenu(Menufunc);
   BuildSubMenus(sub_menu_rotate_id, sub_menu_translate_id, 
		 sub_menu_mann_id, sub_menu_program_id);

   glutSetMenu(menu_id);
   glutAddMenuEntry(S_MENU_ENTRY_ID_10, 10);
   glutAddSubMenu(S_MENU_ENTRY_ID_20, sub_menu_rotate_id);
   glutAddSubMenu(S_MENU_ENTRY_ID_30, sub_menu_translate_id);
   glutAddMenuEntry(S_MENU_ENTRY_ID_40, 40);
   glutAddSubMenu(S_MENU_ENTRY_ID_50, sub_menu_mann_id);
   glutAddSubMenu(S_MENU_ENTRY_ID_60, sub_menu_program_id);
   glutAddMenuEntry(S_MENU_ENTRY_ID_70, 70);
   glutAttachMenu(GLUT_RIGHT_BUTTON);
};

void RebuildMenu()
{
   glutDestroyMenu(menu_id);
   InitMenu();
};

//---Tastatur/Maus/Joystick-Eingabeverarbeitung--------------------------

void DoMove(short xmove, short ymove, short jump)
{
  int	winkel = 0;

  if (spraum->GetViewMann()) winkel = weihmann_id->GetRichtung();
     else winkel = (360 - aktangle[1]) % 360;			//1 = Y
  if ((winkel <= 45) || (winkel > 315)) winkel = 0;
  else if ((winkel > 45) && (winkel <= 135)) winkel = 90;
  else if ((winkel > 135) && (winkel <= 225)) winkel = 180;
  else if ((winkel > 225) && (winkel <= 315)) winkel = 270;
  if (xmove == -1) winkel = (360 + winkel - 90) % 360;		//Links
  if (xmove ==  1) winkel = (360 + winkel + 90) % 360;		//Rechts
  if (ymove == -1) winkel = (360 + winkel - 180) % 360;		//Unten
  if (ymove ==  1) winkel = winkel;				//Oben
  xmove = 0; ymove = 0;
  if (winkel == 270) xmove = -1;		//Links
  if (winkel ==  90) xmove =  1;		//Rechts
  if (winkel == 180) ymove = -1;		//Unten
  if (winkel ==   0) ymove =  1;		//Oben
  winkel = (360 + winkel - 180) % 360;		//fuer Richtung
  if (jump == 1)
	weihmann_id->AddPos(spraum, 0, 0, 1);
    else {
  		weihmann_id->NeuRichtung(winkel);			
  		weihmann_id->AddPos(spraum, xmove, ymove, 0);
	 };
};

static void joystickfunc(unsigned int buttonMask, int x, int y, int z)
{
  int   movex, movey, sense;

  sense = 5;				// Minimum-Differenz zwischen
					// zweier Mauspunkte zum Bewegen
  movex = x - joystickx;				//Pos-Differenz
  movey = y - joysticky;
  if ( abs(movex) >= abs(movey) ) 
	movey = 0; else movex = 0;			//nicht diagonal
  if (buttonMask & GLUT_JOYSTICK_BUTTON_A)
	DoMove(0, 0, 1);
  if (movex < -sense) DoMove(-1,  0, 0);		//Links
  if (movex >  sense) DoMove( 1,  0, 0);		//Rechts
  if (movey < -sense) DoMove( 0, -1, 0);		//Unten
  if (movey >  sense) DoMove( 0,  1, 0);		//Oben
  joystickx = x; joysticky = y;
  glutPostRedisplay();
}

static void mousefunc(int button, int state, int x, int y)
{
  mousex = x; mousey = y;
  if ((button == GLUT_MIDDLE_BUTTON) && (state == GLUT_DOWN))
	DoMove(0, 0, 1);
}

static void mousemotionfunc(int x, int y)
{
  const int 	anglescale = 5;
  const float	transscale = 20.0f;
  char		axis1, axis2;
  int		movex, movey, sense, sign;

  sense = MOUSE_SENSE;			// Minimum-Differenz zwischen
					// zweier Mauspunkte zum Bewegen
  movex = x - mousex;					//Pos-Differenz
  movey = y - mousey;
  if ((abs(movex) > sense) || (abs(movey) > sense))
  	{ mousex = x; mousey = y; };
  if (modifmodemouse == play)
    {
	if (abs(movex) >= abs(movey)) 
		movey = 0; else movex = 0;		//nicht diagonal
  	if (movex < -sense) DoMove(-1,  0, 0); 		//Links
  	if (movex >  sense) DoMove( 1,  0, 0); 		//Rechts
  	if (movey < -sense) DoMove( 0, -1, 0);		//Unten
  	if (movey >  sense) DoMove( 0,  1, 0);		//Oben
    };
  axis1 = 'a'; axis2 = 'b';				// Zero-Values
  if (modifmodemouse == anglexy) { axis1 = 'y'; axis2 = 'x'; };
  if (modifmodemouse == anglexz) { axis1 = 'z'; axis2 = 'x'; };
  if (modifmodemouse == angleyz) { axis1 = 'z'; axis2 = 'y'; };
  if (axis1 == 'z') movex = -movex;
  DoRotate(axis1, movex / anglescale); 
  DoRotate(axis2, movey / anglescale);
  axis1 = 'a'; axis2 = 'b';				// Zero-Values
  if (modifmodemouse == transxy) { axis1 = 'x'; axis2 = 'y'; };
  if (modifmodemouse == transxz) { axis1 = 'x'; axis2 = 'z'; };
  if (modifmodemouse == transyz) { axis1 = 'z'; axis2 = 'y'; };
  if (axis2 == 'y') movey = -movey;
  DoTranslation(axis1, movex / transscale); 
  DoTranslation(axis2, movey / transscale);
  if (modifmodemouse == scroll) 
    {
	if (abs(movex) > abs(movey)) movey = 0;	// Scroll xor Translation
	  else			     movex = 0;
	sign = (movey > 0) ? 1 : -1;
	sign = (abs(movey) > sense) ? sign : 0;
	if (prog_mode == mfileview) fileview->Scroll(sign);
	if (prog_mode == mhighscore) highscore->Scroll(sign);
	DoTranslation('x', movex / transscale);
    };
  glutPostRedisplay();
}

static void mousepassivefunc(int x, int y)
{
  mousex = x; mousey = y;
};

static void specialkey(int key, int x, int y)
{
  const float transscale = 10.0f;

  switch (key) {
     case GLUT_KEY_END	 : DoMove( 0,  0, 1); break;
     case GLUT_KEY_LEFT  : if ((prog_mode == mfileview) || 
			       (prog_mode == mhighscore))
				  DoTranslation('x', transscale);
			     else DoMove(-1,  0, 0); 
			   break;
     case GLUT_KEY_RIGHT : if ((prog_mode == mfileview) || 
			       (prog_mode == mhighscore))
				  DoTranslation('x', -transscale);
			     else DoMove( 1,  0, 0);
			   break;
     case GLUT_KEY_UP    : DoMove( 0, -1, 0); break;
     case GLUT_KEY_DOWN  : DoMove( 0,  1, 0); break;
     case GLUT_KEY_PAGE_UP :	if (prog_mode == mfileview)
				  fileview->Scroll(-1); 
				else if (prog_mode == mhighscore)
				  highscore->Scroll(-1); 
				break;
     case GLUT_KEY_PAGE_DOWN : 	if (prog_mode == mfileview)
				  fileview->Scroll(+1); 
				else if (prog_mode == mhighscore)
				  highscore->Scroll(+1); 
				break;
     }
   glutPostRedisplay();
}

static void key(unsigned char k, int x, int y)
{
  short mult;

  switch (k) {
     case 'v' : modifmodekey = trans; 		break;
     case 'd' : modifmodekey = angle; 		break;
     case 'n' : CreateFigure();			break;
     case 'r' : modifmodekey = reshape; 	break;
     case 's' : SwitchFigure();			break;
     case 'p' : weihmann_id->TriggerPunktAnz(); break;
     case 'b' : if (prog_mode == mgame)
		  spraum->TriggerViewMann(); 	break;
     case 'l' : ShowFile("gpl.txt");		break; // license
     case 'h' : ShowFile(S_NAME_HELPFILE);	break; // help
     case 27  : if (prog_mode == mfileview)
			SwitchToGame();
		else 	spraum->SwitchEnd(0);	break;
     }
  if ( isupper(k) ) mult = -1; else mult = 1;
  k = tolower(k);
  if (k == 'i') mult = 0;
  if (modifmodekey == angle) DoRotate(k, mult * 30);
  if (modifmodekey == trans) DoTranslation(k, mult * 1);
  if (modifmodekey == reshape)
    switch (k) {
     case 'x' : if (aktwidth > -10*mult) 
			aktwidth = (aktwidth + mult*10); break;
     case 'y' : if (aktheight > -10*mult) 
			aktheight = (aktheight + mult*10); break; 
     case 'i' : aktwidth = ANFWIDTH; aktheight = ANFHEIGHT; break;
    }  
  if (reshape) glutReshapeWindow(aktwidth, aktheight);
  glutPostRedisplay();
}

//---Hauptprogramm---------------------------------------------

int main(int argc, char** argv)
{
   short laenge=0;
   char  *vorname=0, *nachname=0;

   cout << S_LICENSE_TEXT_1 << endl;
   cout << S_LICENSE_TEXT_2 << endl;
   cout << S_LICENSE_TEXT_3 << endl;
   cout << S_LICENSE_TEXT_4 << endl;
   cout << S_LICENSE_TEXT_5 << endl;

   if (argc >= 1) highscorefile = argv[1];
   if (argc >= 2) vorname = argv[2];
   if (argc >= 3) nachname = argv[3]; else nachname = "RandomXmas 1.14";
   if (vorname) laenge = strlen(vorname);
   if (nachname) laenge = laenge + strlen(nachname) + 1;
   full_name = new char[ laenge ];
   if (vorname) 
	{ strcat(full_name, vorname); strcat(full_name, " "); };
   if (nachname) strcat(full_name, nachname);
   initVars();

   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
   glutInitWindowPosition (100, 100);
   glutCreateWindow (S_WINDOW_NAME);
   init();
   InitMenu();
   initVarsTwo();

   glutDisplayFunc(display);
   glutReshapeFunc(DoReshape);
   glutKeyboardFunc(key);
   glutSpecialFunc(specialkey);
   glutJoystickFunc(joystickfunc, 10);
   glutMouseFunc(mousefunc);
   glutMotionFunc(mousemotionfunc);
   glutPassiveMotionFunc(mousepassivefunc);
   glutReshapeWindow(aktwidth, aktheight);

   glutMainLoop();

   return 0; 
}

