/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <stdio.h>

#include "../include/spielraum.h"

void Spielraum::InitTextur(const char* texfile, Texture* &thistex)
{
  short   usable;
  FILE	  *fpr;

  thistex = NULL;
  fpr = fopen(texfile, "r");
  if (fpr) usable = 1;
    else   usable = 0;
  if (fpr) fclose(fpr);
  if (usable)  
   {
	thistex = new Texture(0.0, 0.0, 0.0);
	thistex->Readbmp(texfile);
   };
};

Spielraum::Spielraum()
{
  int i;

  currentlevel = 1; points = 0; end = 0; viewmann = 0;
  for(i=0; i<NUMOFTEX; i++) texarray[i] = NULL;
  if (USE_TEXTURES)
    {
	InitTextur(FILETEXEBENE,   texarray[0]);
	InitTextur(FILETEXHUEGEL,  texarray[1]);
	InitTextur(FILETEXBAUM,    texarray[2]);
	InitTextur(FILETEXCAP,     texarray[3]);
	InitTextur(FILETEXHEAD,    texarray[4]);
	InitTextur(FILETEXBODY,    texarray[5]);
	InitTextur(FILETEXSHOES,   texarray[6]);
    };
  for(i=0; i < MAXEBENEN-2; i++)
    {
	levels[i] = NULL;
	usedlevels[i] = 0;
    };
  levels[currentlevel - 1] = new Ebene(currentlevel-1, 
				       const_cast<const Texture**>(texarray));
  usedlevels[currentlevel - 1 ] = 1;
};

Spielraum::~Spielraum()
{
   int i;

   for(i=0; i < MAXEBENEN-2; i++)
     {
	if ( levels[i] ) delete levels[i];
	usedlevels[i] = 0;
     };
   for(i=0; i < NUMOFTEX; i++)
   	if (texarray[i]) delete texarray[i];
};

int Spielraum::SwitchLevel(int nextlevel)
{
   int lastlevel;

   if (currentlevel == nextlevel) 
     if (currentlevel+1 < MAXEBENEN) nextlevel++;
       else nextlevel--;
   if ((nextlevel > 0) && (nextlevel < MAXEBENEN-1))
     {
	levels[currentlevel-1]->TrigFahrstuhl();	//Switch off
	if (!(usedlevels[nextlevel-1]))
	  {
	   	levels[nextlevel-1] = new Ebene(nextlevel-1,
					const_cast<const Texture**>(texarray));
	  	usedlevels[nextlevel-1] = 1;
	  }
	  else levels[nextlevel-1]->TrigFahrstuhl();	//Switch On
						
	lastlevel = currentlevel;			//Change
	currentlevel = nextlevel;
     };
   return currentlevel;
};

int Spielraum::GetContent(int neuposx, int neuposy, Feldpunkte* &content)
{
   return levels[currentlevel-1]->IstDortBaum(neuposx, neuposy, content);
};

int Spielraum::GetAbove(int neuposx, int neuposy, Feldpunkte* &content)
{
   return levels[currentlevel-1]->GetDarueber(neuposx, neuposy, content);
};

int Spielraum::GetEnd()
{
   return end;
};

int Spielraum::SwitchEnd(int newpoints)
{
   end = end + 1;
   if (viewmann) TriggerViewMann();
   if (end == 1) points = newpoints;	// else not useful
   return end;
};

int Spielraum::GetPoints()
{
   return points;
};

int Spielraum::GetViewMann()
{
   return viewmann;
};

int Spielraum::TriggerViewMann()
{
   if (viewmann)
     {
	viewmann = 0;
	glLoadIdentity();
	gluLookAt( GLOBAL_VIEW[0], GLOBAL_VIEW[1], GLOBAL_VIEW[2],
		   GLOBAL_VIEW[3], GLOBAL_VIEW[4], GLOBAL_VIEW[5], 
		   GLOBAL_VIEW[6], GLOBAL_VIEW[7], GLOBAL_VIEW[8]);
     }
  else viewmann = 1;
  return viewmann;
};

const Texture** Spielraum::GetTexarray()
{
   return const_cast<const Texture**>(texarray);
};

int Spielraum::GetLevelNum()
{
  return currentlevel;
};

void Spielraum::DrawSpielraum()
{
  	levels[currentlevel-1]->DrawEbene();
};

void Spielraum::DrawLevelNum()
{
  	levels[currentlevel-1]->DrawLevelNum();
};

