/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <GL/glut.h>
#include <stdlib.h>
#include "../include/texturen.h"

//--- globale Konstanten/Variablen -----------------------

const short 	MAXFELD = 4;
int		aktxwinkel, aktywinkel, aktzwinkel;
float		aktxtrans, aktytrans, aktztrans;
int 		aktwidth, aktheight;
int 		trans=0, winkel=1, reshape=0, ende=0;
char		*bildfile;

Texture		*bild;

//***Funktionen, OpenGL-Grundlagen***********************

void initVars()
{
  aktxwinkel = 0; aktywinkel = 0; aktzwinkel = 0;
  aktxtrans = 0; aktytrans = 0; aktztrans = 0;
  aktwidth = 400; aktheight = 400;
};

void initVarsTwo()
{
  bild = new Texture(-7, -3, 0);
  bild->Readbmp(bildfile);
};

//---Ende + Darstellungsfunktion, greift auf die Klassen zu-------

void endeproc()
{ 
   delete bild;
   exit(0);
}

void display(void)
{  
  glPushMatrix();
  glTranslatef(aktxtrans/5, aktytrans/5, aktztrans/5);
  glRotatef(aktxwinkel, 1, 0, 0);
  glRotatef(aktywinkel, 0, 1, 0);
  glRotatef(aktzwinkel, 0, 0, 1);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  if (ende == 0) bild->DrawTexture();
  else endeproc();
  glFlush();
  glPopMatrix();
}

//---Einstellungen fr Darstellung-----------------------------

void init (void)
{
   glClearColor (0.0, 0.0, 0.0, 0.0);
}

void DoReshape(int width, int height)
{
   int minsize, maxsize;

   aktwidth = width; aktheight = height;
   if (width >= height) { minsize = height; maxsize = width; }
     else { minsize = width; maxsize = height; };
   
   glViewport(0, 0, width, height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(60*maxsize/minsize, 1, 1, 15*MAXFELD);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(0, MAXFELD*7/2, MAXFELD*5, 0, MAXFELD, 0, 0, 1, 0);
}

//---Tastatur-Eingabeverarbeitung-----------------------------

static void key(unsigned char k, int x, int y)
{
  switch (k) {
     case 'v' : trans=1; winkel=0; reshape=0; break;
     case 'd' : winkel=1; trans=0; reshape=0; break;
     case 'r' : reshape=1; trans=0; winkel=0; break;
     case 27  : ende = 1; break;
     }
  if (winkel)
   switch (k) {
     case 'x' : aktxwinkel = (aktxwinkel + 30) % 360; break;
     case 'X' : aktxwinkel = (aktxwinkel - 30) % 360; break;
     case 'y' : aktywinkel = (aktywinkel + 30) % 360; break;
     case 'Y' : aktywinkel = (aktywinkel - 30) % 360; break;
     case 'z' : aktzwinkel = (aktzwinkel + 30) % 360; break;
     case 'Z' : aktzwinkel = (aktzwinkel - 30) % 360; break;     
     case 'i' : aktxwinkel = 0; aktywinkel = 0; aktzwinkel = 0; break; 
   }
  if (trans)
   switch (k) {
     case 'x' : aktxtrans = (aktxtrans + 1) ; break;
     case 'X' : aktxtrans = (aktxtrans - 1) ; break;
     case 'y' : aktytrans = (aktytrans + 1) ; break;
     case 'Y' : aktytrans = (aktytrans - 1) ; break;
     case 'z' : aktztrans = (aktztrans + 1) ; break;
     case 'Z' : aktztrans = (aktztrans - 1) ; break;     
     case 'i' : aktxtrans = 0; aktytrans = 0; aktztrans = 0; break;
    }
  if (reshape)
    switch (k) {
     case 'x' : aktwidth = (aktwidth + 10) ; break;
     case 'X' : if (aktwidth>10) aktwidth = (aktwidth - 10) ; break;
     case 'y' : aktheight = (aktheight + 10) ; break;
     case 'Y' : if (aktheight>10) aktheight = (aktheight - 10) ; break;    
     case 'i' : aktwidth = 400; aktheight = 400; break;
    }
  if (reshape) glutReshapeWindow(aktwidth, aktheight);
  glutPostRedisplay();
}

//---Hauptprogramm---------------------------------------------

int main(int argc, char** argv)
{
   if (argc >= 1) bildfile = argv[1];
   initVars();

   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowPosition (100, 100);
   glutCreateWindow ("Image");
   init();
   initVarsTwo();

   glutDisplayFunc(display);
   glutReshapeFunc(DoReshape);
   glutKeyboardFunc(key);
   glutReshapeWindow(aktwidth, aktheight);

   glutMainLoop();

   return 0; 
}

