/*  This program is a game with a partly randomized scenery
    Copyright (C) 2002   Stefan Ziegler
    This file is part of RandomXmas v1.14.

    RandomXmas v1.14 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RandomXmas v1.14 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy (gpl.txt) of the GNU General Public
    License along with RandomXmas v1.14; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

While executing the program, you get the license text by pressing the key 'l'.
E-mail address: stefan.ziegler@s1998.tu-chemnitz.de
For additional and/or more actual information see file COPYING in the root
directory of this package.
*/

#include <stdio.h>
#include <GL/glu.h>

#include "../include/texturen.h"
#include "../include/language.h"

Texture::Texture(float newxpos, float newypos, float newzpos)
{
   int i;

   imagergba = NULL;
   texnumber = 0;
   xsize = 0; ysize = 0;
   xpos = newxpos; ypos = newypos; zpos = newzpos;
   for(i=0; i<256; i++)
     {
	colortable[i][0] = 0; colortable[i][1] = 0;
	colortable[i][2] = 0; colortable[i][3] = 0;
     };
};

Texture::~Texture()
{
   if (imagergba) delete imagergba;
   imagergba = NULL;
};

void Texture::Test_print_error(int boolvar, 
			unsigned char &error, const char* errortext)
{
   if (boolvar)
     {
	error = 1;
	fprintf(stderr, errortext);
     };
};

int intpower(int x, int y)
{
   int i, res;

   res = 1;
   for(i=1; i<y; i++) res = res * x;

   return res;
};

void Texture::Readbmp(const char* filename)
{
   FILE *fpr;
   unsigned char *header, *line, error=0;
   int	number, filesize, colortablesize, colortablebegin, colordepth;
   int  linesize, y, x;

   fpr = fopen(filename, "r");
   if (fpr)
     {
	header = new unsigned char[54];
	number = fread(header, 1, 54, fpr);
	Test_print_error(number < 54, error, S_TEXTURE_ERROR_1); //too small
	if (!error)		// Read Header
	  {
		Test_print_error( (header[0] != 'B') && (header[1] != 'M'),
			error, S_TEXTURE_ERROR_2);	// wrong start chars
		filesize = header[5]*256*256*256 + header[4]*256*256;
		filesize = filesize + header[3]*256 + header[2];  // 32 bit
		colortablesize = header[11]*256 + header[10] - 54;
		numofcolors = colortablesize / 4;
		if ((colortablesize == 0) && (error == 0)) 
			numofcolors = 16777216; 		  //2^24
		//printf("Colortablesize: %d, numofcolors: %d\n", 
		//	colortablesize, numofcolors);
		colortablebegin = header[15]*256 + header[14];
		xsize = header[20]*256*256 + header[19]*256 + header[18];
		ysize = header[24]*256*256 + header[23]*256 + header[22];
		Test_print_error(xsize <= 0, error, S_TEXTURE_ERROR_3);
		Test_print_error(ysize <= 0, error, S_TEXTURE_ERROR_4);
		colordepth = header[28];	// per pixel
		//printf("Sizes: F%d C%d X%d Y%d D%d\n", filesize, numofcolors, 
		//	xsize, ysize, colordepth);
		//for(y=0; y<54; y++) printf("%d ", header[y]);
		//printf("\n");
	  };
	delete [] header;

	header = new unsigned char[256*4];
	number = fread(header, 1, colortablesize, fpr);
	Test_print_error(number < colortablesize, error,
		S_TEXTURE_ERROR_5);	// wrong color table size
	if (!error)			// Read Colortable
	  for(y=0; y < colortablesize/4; y++)
	    {
		//printf("%d %x %x %x %x \n", y, header[y*4+0], header[y*4+1], 
		//	header[y*4+2], header[y*4+3]);
		colortable[y][0] = header[y*4+2];
		colortable[y][1] = header[y*4+1];
		colortable[y][2] = header[y*4+0];
		colortable[y][3] = 200;			// 1/5 transparency
		//black = header[y*4+2] + header[y*4+1] + header[y*4+0];
		//if (black == 0) colortable[y][3] = 0;	// transparent
		//  else 	  colortable[y][3] = 255; // no transparence
	    };
	delete [] header;

	y = 0;
	//printf("Read Image\n");
	if (xsize % 4 > 0) linesize = 1;
	  else		   linesize = 0;
	linesize = ((xsize / 4 + linesize) * colordepth / 2);
	//printf("Linesize: %d\n", linesize);
	imagergba = new pixeltype[ysize*xsize];
	if (!imagergba) error = 1;
	line = new unsigned char[linesize];
	while (!error)		// Read image data
	  {
		number = fread(line, 1, linesize, fpr);
		Test_print_error(number < linesize, error, "");
		Test_print_error( feof(fpr), error, "");
		if (y >= ysize) error = error + 2;
		if ((!error) && (colordepth == 8))
		  for(x=0; x<xsize; x++)
		    {
			number = line[x];
			imagergba[y*xsize + x][0] = colortable[number][0];
			imagergba[y*xsize + x][1] = colortable[number][1];
			imagergba[y*xsize + x][2] = colortable[number][2];
			imagergba[y*xsize + x][3] = colortable[number][3];
		    };
		if ((!error) && (colordepth == 4))
		  for(x=0; x<xsize; x++)
		    {
			if (x%2) number = line[x/2] & 0x0F;
			  else   number = (line[x/2] & 0xF0) / 16;
			imagergba[y*xsize + x][0] = colortable[number][0];
			imagergba[y*xsize + x][1] = colortable[number][1];
			imagergba[y*xsize + x][2] = colortable[number][2];
			imagergba[y*xsize + x][3] = colortable[number][3];
		    };
		if ((!error) && (colordepth == 24))
		  for(x=0; x<xsize; x++)
		    {
			imagergba[y*xsize + x][0] = line[x*3 + 2];
			imagergba[y*xsize + x][1] = line[x*3 + 1];
			imagergba[y*xsize + x][2] = line[x*3 + 0];
			imagergba[y*xsize + x][3] = 200;     // no transparence
			//black = line[y*4+2] + line[y*4+1] + line[y*4+0];			
			//if (black == 0) 
			//	imagergba[y*xsize + x][3] = 0;	// transparent
		  	// else	imagergba[y*xsize + x][3] = 255;
							// no transparence
		    };
		if ((!error) && (colordepth == 1))
		  for(x=0; x<xsize; x++)
		    {
			number = intpower(2, 8 - x % 8);
			number = (line[x/8] & number) / number;
			imagergba[y*xsize + x][0] = colortable[number][0];
			imagergba[y*xsize + x][1] = colortable[number][1];
			imagergba[y*xsize + x][2] = colortable[number][2];
			imagergba[y*xsize + x][3] = colortable[number][3];
		    };
		y = y + 1;
	  };
	delete [] line;
	if (error == 1) { if (imagergba) delete [] imagergba;
			  imagergba = NULL; }
	  else LoadTexture();
	if (error == 1) 
	  {
		fprintf(stderr, S_TEXTURE_ERROR_6);  // There were errors
		fprintf(stderr, filename); fprintf(stderr, "\n");
	  };
	if (fpr) fclose(fpr);
     };
};

void Texture::LoadTexture()
{
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LESS);
   //glShadeModel(GL_FLAT);
   //printf("LoadTexture\n");

   glEnable(GL_TEXTURE_2D);
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
   glGenTextures(1, &texnumber);
   glBindTexture(GL_TEXTURE_2D, texnumber);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
			GL_NEAREST_MIPMAP_NEAREST);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
			GL_NEAREST_MIPMAP_NEAREST);
   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
   glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
   glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
   glEnable(GL_TEXTURE_GEN_S);
   glEnable(GL_TEXTURE_GEN_T);                                                     
   gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, xsize, ysize, GL_RGBA,
		GL_UNSIGNED_BYTE, &imagergba[0][0]);
   glBindTexture(GL_TEXTURE_2D, 0);
};

void Texture::DrawTexture() const
{
   float xf, yf;

   glTranslatef(xpos, ypos, zpos);
   if (imagergba)
     {
	xf = xsize; yf = ysize;
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);                                                     
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glBindTexture(GL_TEXTURE_2D, texnumber);
	glBegin(GL_QUADS);
	  glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 0.0, 0.0);
	  glTexCoord2f(0.0, 1.0); glVertex3f(0.0, yf/20, 0.0);
	  glTexCoord2f(1.0, 1.0); glVertex3f(xf/20, yf/20, 0.0);
	  glTexCoord2f(1.0, 0.0); glVertex3f(xf/20, 0.0, 0.0);
	glEnd();
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);                                                     
     };
   glTranslatef(-xpos, -ypos, -zpos);
};

GLuint Texture::GetTexNumber() const
// glBindTexture with this number, then draw polygon like in DrawTexture.
{
   return texnumber;
};

